@@ -30,20 +30,6 @@
 
 #include "d3d10.h"
 
-BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
-{
-    switch (fdwReason)
-    {
-        case DLL_WINE_PREATTACH:
-            return FALSE;    /* prefer native version */
-        case DLL_PROCESS_ATTACH:
-            DisableThreadLibraryCalls(hinstDLL);
-            break;
-    }
-
-   return TRUE;
-}
-
 /***********************************************************************
  * D3DX10CheckVersion
  *
@@ -51,7 +37,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
  */
 BOOL WINAPI D3DX10CheckVersion(UINT d3dsdkvers, UINT d3dxsdkvers)
 {
-    if ((d3dsdkvers == D3D10_SDK_VERSION) && (d3dxsdkvers == 41))
+    if ((d3dsdkvers == D3D10_SDK_VERSION) && (d3dxsdkvers == 42))
         return TRUE;
 
     return FALSE;
