@@ -31,6 +31,7 @@
 #include "winnls.h"
 #include "wine/list.h"
 #include "wine/rbtree.h"
+#include "wine/debug.h"
 
 #include "cvconst.h"
 
@@ -182,11 +183,13 @@ static inline BOOL symt_check_tag(const struct symt* s, enum SymTagEnum tag)
     return s && s->tag == tag;
 }
 
+typedef ULONG_PTR symref_t;
+
 /* lexical tree */
 struct symt_block
 {
     struct symt                 symt;
-    struct symt*                container;      /* block, or func */
+    symref_t                    container;      /* block, or func */
     struct vector               vchildren;      /* sub-blocks & local variables */
     unsigned                    num_ranges;
     struct addr_range           ranges[];
@@ -202,9 +205,9 @@ struct symt_module /* in fact any of .exe, .dll... */
 struct symt_compiland
 {
     struct symt                 symt;
-    struct symt_module*         container;      /* symt_module */
+    symref_t                    container;      /* symt_module */
     ULONG_PTR                   address;
-    unsigned                    source;
+    const char                 *filename;
     struct vector               vchildren;      /* global variables & functions */
     void*                       user;           /* when debug info provider needs to store information */
 };
@@ -214,8 +217,8 @@ struct symt_data
     struct symt                 symt;
     struct hash_table_elt       hash_elt;       /* if global symbol */
     enum DataKind               kind;
-    struct symt*                container;
-    struct symt*                type;
+    symref_t                    container;
+    symref_t                    type;
     union                                       /* depends on kind */
     {
         /* DataIs{Global, FileStatic, StaticLocal}:
@@ -291,11 +294,12 @@ struct symt_function
 {
     struct symt                 symt;           /* SymTagFunction or SymTagInlineSite */
     struct hash_table_elt       hash_elt;       /* if global symbol, inline site */
-    struct symt*                container;      /* compiland (for SymTagFunction) or function (for SymTagInlineSite) */
-    struct symt*                type;           /* points to function_signature */
+    symref_t                    container;      /* compiland (for SymTagFunction) or function (for SymTagInlineSite) */
+    symref_t                    type;           /* points to function_signature */
     struct vector               vlines;
     struct vector               vchildren;      /* locals, params, blocks, start/end, labels, inline sites */
     struct symt_function*       next_inlinesite;/* linked list of inline sites in this function */
+    DWORD_PTR                   user;           /* free to use by debug info backends */
     unsigned                    num_ranges;
     struct addr_range           ranges[];
 };
@@ -304,7 +308,7 @@ struct symt_hierarchy_point
 {
     struct symt                 symt;           /* either SymTagFunctionDebugStart, SymTagFunctionDebugEnd, SymTagLabel */
     struct hash_table_elt       hash_elt;       /* if label (and in compiland's hash table if global) */
-    struct symt*                parent;         /* symt_function or symt_compiland */
+    symref_t                    container;      /* symt_function or symt_compiland */
     struct location             loc;
 };
 
@@ -312,8 +316,8 @@ struct symt_public
 {
     struct symt                 symt;
     struct hash_table_elt       hash_elt;
-    struct symt*                container;      /* compiland */
-    BOOL is_function;
+    symref_t                    container;      /* compiland */
+    BOOL                        is_function;
     ULONG_PTR                   address;
     ULONG_PTR                   size;
 };
@@ -322,7 +326,7 @@ struct symt_thunk
 {
     struct symt                 symt;
     struct hash_table_elt       hash_elt;
-    struct symt*                container;      /* compiland */
+    symref_t                    container;      /* compiland */
     ULONG_PTR                   address;
     ULONG_PTR                   size;
     THUNK_ORDINAL               ordinal;        /* FIXME: doesn't seem to be accessible */
@@ -386,7 +390,7 @@ struct symt_typedef
 {
     struct symt                 symt;
     struct hash_table_elt       hash_elt;
-    struct symt*                type;
+    symref_t                    type;
 };
 
 struct symt_udt
@@ -411,6 +415,7 @@ enum format_info
     DFI_MACHO,
     DFI_DWARF,
     DFI_PDB,
+    DFI_OLD_PDB,
     DFI_LAST
 };
 
@@ -433,6 +438,17 @@ struct module_format_vtable
 {
     /* module handling */
     void                        (*remove)(struct module_format* modfmt);
+
+    /* index management */
+    enum method_result          (*request_symref_t)(struct module_format *modfmt, symref_t ref, IMAGEHLP_SYMBOL_TYPE_INFO req, void *data);
+    enum method_result          (*lookup_by_address)(struct module_format *modfmt, DWORD_PTR address, symref_t *symref);
+    enum method_result          (*lookup_by_name)(struct module_format *modfmt, const char *name, symref_t *symref);
+    enum method_result          (*enumerate_symbols)(struct module_format *modfmt, const WCHAR *match, BOOL (*cb)(symref_t, const char *, void *), void *user);
+
+    /* types management */
+    enum method_result          (*find_type)(struct module_format *modfmt, const char *name, symref_t *ref);
+    enum method_result          (*enumerate_types)(struct module_format *modfmt, BOOL (*cb)(symref_t, const char *, void*), void *user);
+
     /* stack walk */
     void                        (*loc_compute)(const struct module_format* modfmt,
                                                const struct symt_function* func,
@@ -444,6 +460,8 @@ struct module_format_vtable
                                                      struct lineinfo_t *line_info, BOOL forward);
     enum method_result          (*enumerate_lines)(struct module_format *modfmt, const WCHAR* compiland_regex,
                                                    const WCHAR *source_file_regex, PSYM_ENUMLINES_CALLBACK cb, void *user);
+    enum method_result          (*get_line_from_inlined_address)(struct module_format *modfmt, struct symt_function *inlined,
+                                                                 DWORD64 address, struct lineinfo_t *line_info);
 
     /* source files information */
     enum method_result          (*enumerate_sources)(struct module_format *modfmt, const WCHAR *sourcefile_regex,
@@ -460,7 +478,8 @@ struct module_format
         struct elf_module_info*         elf_info;
         struct dwarf2_module_info_s*    dwarf2_info;
         struct pe_module_info*          pe_info;
-        struct macho_module_info*	macho_info;
+        struct macho_module_info*       macho_info;
+        struct old_pdb_module_info*     old_pdb_info;
         struct pdb_module_info*         pdb_info;
     } u;
 };
@@ -486,6 +505,12 @@ struct module
     /* specific information for debug types */
     struct module_format*       format_info[DFI_LAST];
     unsigned                    debug_format_bitmask;
+    /* Hack for fast symdef deref...
+     * Note: if ever we need another backend with dedicated symref_t support,
+     * we could always use the 3 non-zero lower bits of symref_t to match a
+     * debug backend.
+     */
+    struct module_format       *ops_symref_modfmt;
 
     /* memory allocation pool */
     struct pool                 pool;
@@ -809,14 +834,9 @@ extern BOOL         pe_load_debug_directory(const struct process* pcs,
                                             const IMAGE_DEBUG_DIRECTORY* dbg, int nDbg);
 extern DWORD        msc_get_file_indexinfo(void* image, const IMAGE_DEBUG_DIRECTORY* dbgdir, DWORD size,
                                            SYMSRV_INDEX_INFOW* info);
-struct pdb_cmd_pair {
-    const char*         name;
-    DWORD*              pvalue;
-};
-extern BOOL pdb_virtual_unwind(struct cpu_stack_walk *csw, DWORD_PTR ip,
-    union ctx *context, struct pdb_cmd_pair *cpair);
-extern DWORD pdb_get_file_indexinfo(void* image, DWORD size, SYMSRV_INDEX_INFOW* info);
-extern DWORD dbg_get_file_indexinfo(void* image, DWORD size, SYMSRV_INDEX_INFOW* info);
+extern DWORD        pdb_get_file_indexinfo(void* image, DWORD size, SYMSRV_INDEX_INFOW* info);
+extern DWORD        dbg_get_file_indexinfo(void* image, DWORD size, SYMSRV_INDEX_INFOW* info);
+extern BOOL         old_pdb_virtual_unwind(struct cpu_stack_walk *csw, DWORD_PTR ip, union ctx *context);
 
 /* path.c */
 extern BOOL         path_find_symbol_file(const struct process *pcs, const struct module *module,
@@ -829,6 +849,14 @@ extern BOOL search_unix_path(const WCHAR *name, const WCHAR *path, BOOL (*match)
 extern const WCHAR* file_name(const WCHAR* str);
 extern const char* file_nameA(const char* str);
 
+/* pdb.c */
+extern BOOL         pdb_init_modfmt(const struct process *pcs, const struct msc_debug_info *msc_dbg,
+                                    const WCHAR *filename, BOOL *has_linenumber_info);
+extern BOOL         pdb_virtual_unwind(struct cpu_stack_walk *csw, DWORD_PTR ip, union ctx *context);
+struct _PDB_FPO_DATA;
+extern BOOL         pdb_fpo_unwind_parse_cmd_string(struct cpu_stack_walk* csw, struct _PDB_FPO_DATA* fpoext,
+                                                    const char* cmd, WOW64_CONTEXT *context);
+
 /* pe_module.c */
 extern BOOL         pe_load_nt_header(HANDLE hProc, DWORD64 base, IMAGE_NT_HEADERS* nth, BOOL* is_builtin);
 extern struct module*
@@ -851,6 +879,7 @@ extern BOOL         pe_has_buildid_debug(struct image_file_map *fmap, GUID *guid
 extern unsigned     source_new(struct module* module, const char* basedir, const char* source);
 extern const char*  source_get(const struct module* module, unsigned idx);
 extern int          source_rb_compare(const void *key, const struct wine_rb_entry *entry);
+extern char        *source_build_path(const char *base, const char *name);
 
 /* stabs.c */
 typedef void (*stabs_def_cb)(struct module* module, ULONG_PTR load_offset,
@@ -891,47 +920,48 @@ extern struct symt_ht*
 extern struct symt_module*
                     symt_new_module(struct module* module);
 extern struct symt_compiland*
-                    symt_new_compiland(struct module* module, unsigned src_idx);
+                    symt_new_compiland(struct module* module, const char *filename);
 extern struct symt_public*
-                    symt_new_public(struct module* module, 
-                                    struct symt_compiland* parent, 
+                    symt_new_public(struct module* module,
+                                    struct symt_compiland* parent,
                                     const char* typename,
                                     BOOL is_function,
                                     ULONG_PTR address,
                                     unsigned size);
 extern struct symt_data*
-                    symt_new_global_variable(struct module* module, 
+                    symt_new_global_variable(struct module* module,
                                              struct symt_compiland* parent,
                                              const char* name, unsigned is_static,
                                              struct location loc, ULONG_PTR size,
-                                             struct symt* type);
+                                             symref_t type);
 extern struct symt_function*
                     symt_new_function(struct module* module,
                                       struct symt_compiland* parent,
                                       const char* name,
                                       ULONG_PTR addr, ULONG_PTR size,
-                                      struct symt* type);
+                                      symref_t type, DWORD_PTR user);
 extern struct symt_function*
                     symt_new_inlinesite(struct module* module,
                                         struct symt_function* func,
                                         struct symt* parent,
                                         const char* name,
-                                        struct symt* type,
+                                        symref_t type,
+                                        DWORD_PTR user,
                                         unsigned num_ranges);
 extern void         symt_add_func_line(struct module* module,
                                        struct symt_function* func, 
                                        unsigned source_idx, int line_num, 
                                        ULONG_PTR offset);
 extern struct symt_data*
-                    symt_add_func_local(struct module* module, 
-                                        struct symt_function* func, 
+                    symt_add_func_local(struct module* module,
+                                        struct symt_function* func,
                                         enum DataKind dt, const struct location* loc,
                                         struct symt_block* block,
-                                        struct symt* type, const char* name);
+                                        symref_t, const char* name);
 extern struct symt_data*
                     symt_add_func_constant(struct module* module,
                                            struct symt_function* func, struct symt_block* block,
-                                           struct symt* type, const char* name, VARIANT* v);
+                                           symref_t, const char* name, VARIANT* v);
 extern struct symt_block*
                     symt_open_func_block(struct module* module,
                                          struct symt_function* func,
@@ -955,14 +985,31 @@ extern struct symt_thunk*
 extern struct symt_data*
                     symt_new_constant(struct module* module,
                                       struct symt_compiland* parent,
-                                      const char* name, struct symt* type,
+                                      const char* name, symref_t type,
                                       const VARIANT* v);
 extern struct symt_hierarchy_point*
                     symt_new_label(struct module* module,
                                    struct symt_compiland* compiland,
                                    const char* name, ULONG_PTR address);
-extern struct symt* symt_index2ptr(struct module* module, DWORD id);
-extern DWORD        symt_ptr2index(struct module* module, const struct symt* sym);
+static inline BOOL  symt_is_symref_ptr(symref_t ref) {return (ref & 3) == 0;}
+static inline symref_t
+                    symt_ptr_to_symref(const struct symt *symt) {return (ULONG_PTR)symt;}
+static inline struct symt*
+                    _symt_symref_to_ptr(const char *file, unsigned lineno, symref_t symref)
+{
+    if (!symt_is_symref_ptr(symref))
+    {
+        MESSAGE("%s:%u can't convert symref to ptr\n", file, lineno);
+        return NULL;
+    }
+    return (struct symt*)symref;
+}
+/* this function shall be used with care as not all symref:s are actual pointers */
+#define SYMT_SYMREF_TO_PTR(s) _symt_symref_to_ptr(__FILE__, __LINE__, (s))
+extern symref_t     symt_index_to_symref(struct module* module, DWORD id);
+extern DWORD        symt_symref_to_index(struct module* module, symref_t sym);
+static inline DWORD symt_ptr_to_index(struct module *module, const struct symt *symt) {return symt_symref_to_index(module, symt_ptr_to_symref(symt));}
+
 extern struct symt_custom*
                     symt_new_custom(struct module* module, const char* name,
                                     DWORD64 addr, DWORD size);
@@ -970,8 +1017,14 @@ extern BOOL         lineinfo_set_nameA(struct process* pcs, struct lineinfo_t* i
 
 /* type.c */
 extern void         symt_init_basic(struct module* module);
+extern BOOL         symt_get_info_raw(struct module* module, const struct symt* type,
+                                      IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
 extern BOOL         symt_get_info(struct module* module, const struct symt* type,
                                   IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
+extern BOOL         symt_get_info_from_index(struct module* module, DWORD index,
+                                             IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
+extern BOOL         symt_get_info_from_symref(struct module* module, symref_t type,
+                                              IMAGEHLP_SYMBOL_TYPE_INFO req, void* pInfo);
 extern struct symt_basic*
                     symt_get_basic(enum BasicType, unsigned size);
 extern struct symt_udt*
@@ -979,10 +1032,10 @@ extern struct symt_udt*
                                  unsigned size, enum UdtKind kind);
 extern BOOL         symt_set_udt_size(struct module* module,
                                       struct symt_udt* type, unsigned size);
-extern BOOL         symt_add_udt_element(struct module* module, 
-                                         struct symt_udt* udt_type, 
+extern BOOL         symt_add_udt_element(struct module* module,
+                                         struct symt_udt* udt_type,
                                          const char* name,
-                                         struct symt* elt_type, unsigned offset, 
+                                         symref_t elt_type, unsigned offset,
                                          unsigned bit_offset, unsigned bit_size);
 extern struct symt_enum*
                     symt_new_enum(struct module* module, const char* typename,
@@ -1001,11 +1054,11 @@ extern BOOL         symt_add_function_signature_parameter(struct module* module,
                                                           struct symt_function_signature* sig,
                                                           struct symt* param);
 extern struct symt_pointer*
-                    symt_new_pointer(struct module* module, 
+                    symt_new_pointer(struct module* module,
                                      struct symt* ref_type,
                                      ULONG_PTR size);
 extern struct symt_typedef*
-                    symt_new_typedef(struct module* module, struct symt* ref, 
+                    symt_new_typedef(struct module* module, symref_t ref,
                                      const char* name);
 extern struct symt_function*
                     symt_find_lowest_inlined(struct symt_function* func, DWORD64 addr);
@@ -1042,14 +1095,3 @@ extern struct symt_function*
 #define IFC_DEPTH_MASK   0x3FFFFFFF
 #define IFC_MODE(x)      ((x) & ~IFC_DEPTH_MASK)
 #define IFC_DEPTH(x)     ((x) & IFC_DEPTH_MASK)
-
-/* temporary helpers for PDB rewriting */
-struct _PDB_FPO_DATA;
-extern BOOL pdb_fpo_unwind_parse_cmd_string(struct cpu_stack_walk* csw, struct _PDB_FPO_DATA* fpoext,
-                                            const char* cmd, struct pdb_cmd_pair* cpair);
-extern BOOL pdb_old_virtual_unwind(struct cpu_stack_walk *csw, DWORD_PTR ip,
-                                   union ctx *context, struct pdb_cmd_pair *cpair);
-struct pdb_reader;
-extern BOOL pdb_hack_get_main_info(struct module_format *modfmt, struct pdb_reader **pdb, unsigned *fpoext_stream);
-extern void pdb_reader_dispose(struct pdb_reader *pdb);
-extern struct pdb_reader *pdb_hack_reader_init(struct module *module, HANDLE file, const IMAGE_SECTION_HEADER *sections, unsigned num_sections);
