@@ -34,7 +34,7 @@
 
 #define DS_MAX_CHANNELS 6
 
-extern int ds_hel_buflen DECLSPEC_HIDDEN;
+extern int ds_hel_buflen;
 
 /*****************************************************************************
  * Predeclare the interface implementation structures
@@ -43,14 +43,14 @@ typedef struct IDirectSoundBufferImpl        IDirectSoundBufferImpl;
 typedef struct DirectSoundDevice             DirectSoundDevice;
 
 /* dsound_convert.h */
-typedef float (*bitsgetfunc)(const IDirectSoundBufferImpl *, DWORD, DWORD);
+typedef float (*bitsgetfunc)(const IDirectSoundBufferImpl *, BYTE *, DWORD);
 typedef void (*bitsputfunc)(const IDirectSoundBufferImpl *, DWORD, DWORD, float);
-extern const bitsgetfunc getbpp[5] DECLSPEC_HIDDEN;
-void putieee32(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void putieee32_sum(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void mixieee32(float *src, float *dst, unsigned samples) DECLSPEC_HIDDEN;
+extern const bitsgetfunc getbpp[5];
+void putieee32(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void putieee32_sum(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void mixieee32(float *src, float *dst, unsigned samples);
 typedef void (*normfunc)(const void *, void *, unsigned);
-extern const normfunc normfunctions[4] DECLSPEC_HIDDEN;
+extern const normfunc normfunctions[4];
 
 typedef struct _DSVOLUMEPAN
 {
@@ -80,7 +80,7 @@ struct DirectSoundDevice
     DWORD                       writelead, buflen, ac_frames, frag_frames, playpos, pad, stopped;
     int                         nrofbuffers;
     IDirectSoundBufferImpl**    buffers;
-    RTL_RWLOCK                  buffer_list_lock;
+    SRWLOCK                     buffer_list_lock;
     CRITICAL_SECTION            mixlock;
     IDirectSoundBufferImpl     *primary;
     DWORD                       speaker_config;
@@ -90,6 +90,7 @@ struct DirectSoundDevice
     int                         lfe_channel;
     float *tmp_buffer, *cp_buffer;
     DWORD                       tmp_buffer_len, cp_buffer_len;
+    CO_MTA_USAGE_COOKIE         mta_cookie;
 
     DSVOLUMEPAN                 volpan;
 
@@ -119,8 +120,8 @@ typedef struct BufferMemory
 
 HRESULT DirectSoundDevice_AddBuffer(
     DirectSoundDevice * device,
-    IDirectSoundBufferImpl * pDSB) DECLSPEC_HIDDEN;
-void DirectSoundDevice_RemoveBuffer(DirectSoundDevice * device, IDirectSoundBufferImpl * pDSB) DECLSPEC_HIDDEN;
+    IDirectSoundBufferImpl * pDSB);
+void DirectSoundDevice_RemoveBuffer(DirectSoundDevice * device, IDirectSoundBufferImpl * pDSB);
 
 /*****************************************************************************
  * IDirectSoundBuffer implementation structure
@@ -136,12 +137,11 @@ struct IDirectSoundBufferImpl
     LONG                        ref, refn, ref3D, refiks;
     /* IDirectSoundBufferImpl fields */
     DirectSoundDevice*          device;
-    RTL_RWLOCK                  lock;
+    SRWLOCK                     lock;
     PWAVEFORMATEX               pwfx;
     BufferMemory*               buffer;
     DWORD                       playflags,state,leadin;
-    DWORD                       writelead,buflen;
-    DWORD                       nAvgBytesPerSec;
+    DWORD                       writelead,maxwritelead,buflen;
     DWORD                       freq;
     DSVOLUMEPAN                 volpan;
     DSBUFFERDESC                dsbd;
@@ -153,13 +153,19 @@ struct IDirectSoundBufferImpl
     LONG64                      freqAccNum;
     /* used for mixing */
     DWORD                       sec_mixpos;
-
+    /* Holds a copy of the next 'writelead' bytes, to be used for mixing. This makes it
+     * so that these bytes get played once even if this region of the buffer gets overwritten,
+     * which is more in-line with native DirectSound behavior. */
+    BOOL                        use_committed;
+    LPVOID                      committedbuff;
+    DWORD                       committed_mixpos;
     /* IDirectSoundNotify fields */
     LPDSBPOSITIONNOTIFY         notifies;
     int                         nrofnotifies;
     /* DirectSound3DBuffer fields */
     DS3DBUFFER                  ds3db_ds3db;
     LONG                        ds3db_lVolume;
+    DWORD                       ds3db_freq;
     BOOL                        ds3db_need_recalc;
     /* Used for bit depth conversion */
     int                         mix_channels;
@@ -171,74 +177,75 @@ struct IDirectSoundBufferImpl
     struct list entry;
 };
 
-float get_mono(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel) DECLSPEC_HIDDEN;
-void put_mono2stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_mono2quad(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_stereo2quad(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_mono2surround51(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_stereo2surround51(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_surround512stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_surround712stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
-void put_quad2stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value) DECLSPEC_HIDDEN;
+float get_mono(const IDirectSoundBufferImpl *dsb, BYTE *base, DWORD channel);
+void put_mono2stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_mono2quad(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_stereo2quad(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_mono2surround51(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_stereo2surround51(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_surround512stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_surround712stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
+void put_quad2stereo(const IDirectSoundBufferImpl *dsb, DWORD pos, DWORD channel, float value);
 
 HRESULT secondarybuffer_create(DirectSoundDevice *device, const DSBUFFERDESC *dsbd,
-        IDirectSoundBuffer **buffer) DECLSPEC_HIDDEN;
+        IDirectSoundBuffer **buffer);
 HRESULT IDirectSoundBufferImpl_Duplicate(
     DirectSoundDevice *device,
     IDirectSoundBufferImpl **ppdsb,
-    IDirectSoundBufferImpl *pdsb) DECLSPEC_HIDDEN;
-void secondarybuffer_destroy(IDirectSoundBufferImpl *This) DECLSPEC_HIDDEN;
-extern const IDirectSound3DListenerVtbl ds3dlvt DECLSPEC_HIDDEN;
-extern const IDirectSound3DBufferVtbl ds3dbvt DECLSPEC_HIDDEN;
-extern const IKsPropertySetVtbl iksbvt DECLSPEC_HIDDEN;
+    IDirectSoundBufferImpl *pdsb);
+void secondarybuffer_destroy(IDirectSoundBufferImpl *This);
+BOOL secondarybuffer_is_audible(IDirectSoundBufferImpl *This);
+extern const IDirectSound3DListenerVtbl ds3dlvt;
+extern const IDirectSound3DBufferVtbl ds3dbvt;
+extern const IKsPropertySetVtbl iksbvt;
 
-HRESULT IKsPrivatePropertySetImpl_Create(REFIID riid, void **ppv) DECLSPEC_HIDDEN;
+HRESULT IKsPrivatePropertySetImpl_Create(REFIID riid, void **ppv);
 
 /*******************************************************************************
  */
 
 /* dsound.c */
 
-HRESULT DSOUND_Create(REFIID riid, void **ppv) DECLSPEC_HIDDEN;
-HRESULT DSOUND_Create8(REFIID riid, void **ppv) DECLSPEC_HIDDEN;
-HRESULT IDirectSoundImpl_Create(IUnknown *outer_unk, REFIID riid, void **ppv, BOOL has_ds8) DECLSPEC_HIDDEN;
-void DSOUND_ParseSpeakerConfig(DirectSoundDevice *device) DECLSPEC_HIDDEN;
+HRESULT DSOUND_Create(REFIID riid, void **ppv);
+HRESULT DSOUND_Create8(REFIID riid, void **ppv);
+HRESULT IDirectSoundImpl_Create(IUnknown *outer_unk, REFIID riid, void **ppv, BOOL has_ds8);
+void DSOUND_ParseSpeakerConfig(DirectSoundDevice *device);
 
 /* primary.c */
 
-HRESULT DSOUND_PrimaryDestroy(DirectSoundDevice *device) DECLSPEC_HIDDEN;
-HRESULT DSOUND_PrimaryPlay(DirectSoundDevice *device) DECLSPEC_HIDDEN;
-HRESULT DSOUND_PrimaryStop(DirectSoundDevice *device) DECLSPEC_HIDDEN;
-LPWAVEFORMATEX DSOUND_CopyFormat(LPCWAVEFORMATEX wfex) DECLSPEC_HIDDEN;
-HRESULT DSOUND_ReopenDevice(DirectSoundDevice *device, BOOL forcewave) DECLSPEC_HIDDEN;
+HRESULT DSOUND_PrimaryDestroy(DirectSoundDevice *device);
+HRESULT DSOUND_PrimaryPlay(DirectSoundDevice *device);
+HRESULT DSOUND_PrimaryStop(DirectSoundDevice *device);
+LPWAVEFORMATEX DSOUND_CopyFormat(LPCWAVEFORMATEX wfex);
+HRESULT DSOUND_ReopenDevice(DirectSoundDevice *device, BOOL forcewave);
 HRESULT primarybuffer_create(DirectSoundDevice *device, IDirectSoundBufferImpl **ppdsb,
-    const DSBUFFERDESC *dsbd) DECLSPEC_HIDDEN;
-void primarybuffer_destroy(IDirectSoundBufferImpl *This) DECLSPEC_HIDDEN;
-HRESULT primarybuffer_SetFormat(DirectSoundDevice *device, LPCWAVEFORMATEX wfex) DECLSPEC_HIDDEN;
-LONG capped_refcount_dec(LONG *ref) DECLSPEC_HIDDEN;
+    const DSBUFFERDESC *dsbd);
+void primarybuffer_destroy(IDirectSoundBufferImpl *This);
+HRESULT primarybuffer_SetFormat(DirectSoundDevice *device, LPCWAVEFORMATEX wfex);
+LONG capped_refcount_dec(LONG *ref);
 
 /* duplex.c */
 
-HRESULT DSOUND_FullDuplexCreate(REFIID riid, void **ppv) DECLSPEC_HIDDEN;
+HRESULT DSOUND_FullDuplexCreate(REFIID riid, void **ppv);
 
 /* mixer.c */
-void DSOUND_CheckEvent(const IDirectSoundBufferImpl *dsb, DWORD playpos, int len) DECLSPEC_HIDDEN;
-void DSOUND_RecalcVolPan(PDSVOLUMEPAN volpan) DECLSPEC_HIDDEN;
-void DSOUND_AmpFactorToVolPan(PDSVOLUMEPAN volpan) DECLSPEC_HIDDEN;
-void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb) DECLSPEC_HIDDEN;
-DWORD DSOUND_secpos_to_bufpos(const IDirectSoundBufferImpl *dsb, DWORD secpos, DWORD secmixpos, float *overshot) DECLSPEC_HIDDEN;
+void DSOUND_CheckEvent(const IDirectSoundBufferImpl *dsb, DWORD playpos, int len);
+void DSOUND_RecalcVolPan(PDSVOLUMEPAN volpan);
+void DSOUND_AmpFactorToVolPan(PDSVOLUMEPAN volpan);
+void DSOUND_RecalcFormat(IDirectSoundBufferImpl *dsb);
+DWORD DSOUND_secpos_to_bufpos(const IDirectSoundBufferImpl *dsb, DWORD secpos, DWORD secmixpos, float *overshot);
 
-DWORD CALLBACK DSOUND_mixthread(void *ptr) DECLSPEC_HIDDEN;
+DWORD CALLBACK DSOUND_mixthread(void *ptr);
 
 /* sound3d.c */
 
-void DSOUND_Calc3DBuffer(IDirectSoundBufferImpl *dsb) DECLSPEC_HIDDEN;
+void DSOUND_Calc3DBuffer(IDirectSoundBufferImpl *dsb);
 
 /* capture.c */
  
-HRESULT DSOUND_CaptureCreate(REFIID riid, void **ppv) DECLSPEC_HIDDEN;
-HRESULT DSOUND_CaptureCreate8(REFIID riid, void **ppv) DECLSPEC_HIDDEN;
-HRESULT IDirectSoundCaptureImpl_Create(IUnknown *outer_unk, REFIID riid, void **ppv, BOOL has_dsc8) DECLSPEC_HIDDEN;
+HRESULT DSOUND_CaptureCreate(REFIID riid, void **ppv);
+HRESULT DSOUND_CaptureCreate8(REFIID riid, void **ppv);
+HRESULT IDirectSoundCaptureImpl_Create(IUnknown *outer_unk, REFIID riid, void **ppv, BOOL has_dsc8);
 
 #define STATE_STOPPED   0
 #define STATE_STARTING  1
@@ -246,33 +253,19 @@ HRESULT IDirectSoundCaptureImpl_Create(IUnknown *outer_unk, REFIID riid, void **
 #define STATE_CAPTURING 2
 #define STATE_STOPPING  3
 
-extern CRITICAL_SECTION DSOUND_renderers_lock DECLSPEC_HIDDEN;
-extern CRITICAL_SECTION DSOUND_capturers_lock DECLSPEC_HIDDEN;
-extern struct list DSOUND_capturers DECLSPEC_HIDDEN;
-extern struct list DSOUND_renderers DECLSPEC_HIDDEN;
+extern CRITICAL_SECTION DSOUND_renderers_lock;
+extern struct list DSOUND_renderers;
 
-extern GUID DSOUND_renderer_guids[MAXWAVEDRIVERS] DECLSPEC_HIDDEN;
-extern GUID DSOUND_capture_guids[MAXWAVEDRIVERS] DECLSPEC_HIDDEN;
+extern GUID *DSOUND_renderer_guids;
+extern GUID *DSOUND_capture_guids;
 
-extern const WCHAR wine_vxd_drv[] DECLSPEC_HIDDEN;
+extern const WCHAR wine_vxd_drv[];
 
-void setup_dsound_options(void) DECLSPEC_HIDDEN;
+void setup_dsound_options(void);
 
-HRESULT get_mmdevice(EDataFlow flow, const GUID *tgt, IMMDevice **device) DECLSPEC_HIDDEN;
+HRESULT get_mmdevice(EDataFlow flow, const GUID *tgt, IMMDevice **device);
 
 BOOL DSOUND_check_supported(IAudioClient *client, DWORD rate,
-        DWORD depth, WORD channels) DECLSPEC_HIDDEN;
+        DWORD depth, WORD channels);
 HRESULT enumerate_mmdevices(EDataFlow flow, GUID *guids,
-        LPDSENUMCALLBACKW cb, void *user) DECLSPEC_HIDDEN;
-
-static inline WCHAR *strdupW( const WCHAR *str )
-{
-    size_t size;
-    WCHAR *ret;
-
-    if (!str) return NULL;
-    size = (lstrlenW( str ) + 1) * sizeof(WCHAR);
-    ret = HeapAlloc( GetProcessHeap(), 0, size );
-    if (ret) memcpy( ret, str, size );
-    return ret;
-}
+        LPDSENUMCALLBACKW cb, void *user);
