@@ -81,16 +81,6 @@ typedef struct _ICCVID_Info
     cinepak_info *cvinfo;
 } ICCVID_Info;
 
-static inline void* __WINE_ALLOC_SIZE(1) heap_alloc(size_t size)
-{
-    return HeapAlloc(GetProcessHeap(), 0, size);
-}
-
-static inline BOOL heap_free(void *mem)
-{
-    return HeapFree(GetProcessHeap(), 0, mem);
-}
-
 
 /* ------------------------------------------------------------------------ */
 static unsigned char *in_buffer, uiclip[1024], *uiclp = NULL;
@@ -178,6 +168,10 @@ int x, y;
     }
 }
 
+static inline int get_stride(int width, int depth)
+{
+    return ((depth * width + 31) >> 3) & ~3;
+}
 
 /* ------------------------------------------------------------------------ */
 static void cvid_v4_32(unsigned char *frm, unsigned char *limit, int stride, BOOL inverted,
@@ -358,7 +352,7 @@ static cinepak_info *decode_cinepak_init(void)
     cinepak_info *cvinfo;
     int i;
 
-    cvinfo = heap_alloc( sizeof (cinepak_info) );
+    cvinfo = malloc(sizeof(cinepak_info));
     if( !cvinfo )
         return NULL;
     cvinfo->strip_num = 0;
@@ -379,10 +373,10 @@ static void free_cvinfo( cinepak_info *cvinfo )
 
     for( i=0; i<cvinfo->strip_num; i++ )
     {
-        heap_free(cvinfo->v4_codebook[i]);
-        heap_free(cvinfo->v1_codebook[i]);
+        free(cvinfo->v4_codebook[i]);
+        free(cvinfo->v1_codebook[i]);
     }
-    heap_free( cvinfo );
+    free( cvinfo );
 }
 
 typedef void (*fn_cvid_v1)(unsigned char *frm, unsigned char *limit,
@@ -459,7 +453,7 @@ static void decode_cinepak(cinepak_info *cvinfo, unsigned char *buf, int size,
             break;
         }
 
-    frm_stride = out_width * bpp;
+    frm_stride = get_stride(out_width, bpp * 8);
     frm_ptr = output;
 
     if(frame.length != size)
@@ -486,13 +480,13 @@ static void decode_cinepak(cinepak_info *cvinfo, unsigned char *buf, int size,
 
         for(i = cvinfo->strip_num; i < frame.strips; i++)
             {
-            if((cvinfo->v4_codebook[i] = heap_alloc(sizeof(cvid_codebook) * 260)) == NULL)
+            if((cvinfo->v4_codebook[i] = malloc(sizeof(cvid_codebook) * 260)) == NULL)
                 {
                 ERR("CVID: codebook v4 alloc err\n");
                 return;
                 }
 
-            if((cvinfo->v1_codebook[i] = heap_alloc(sizeof(cvid_codebook) * 260)) == NULL)
+            if((cvinfo->v1_codebook[i] = malloc(sizeof(cvid_codebook) * 260)) == NULL)
                 {
                 ERR("CVID: codebook v1 alloc err\n");
                 return;
@@ -748,14 +742,14 @@ static void ICCVID_dump_BITMAPINFO(const BITMAPINFO * bmi)
     TRACE(
         "planes = %d\n"
         "bpp    = %d\n"
-        "height = %d\n"
-        "width  = %d\n"
+        "height = %ld\n"
+        "width  = %ld\n"
         "compr  = %s\n",
         bmi->bmiHeader.biPlanes,
         bmi->bmiHeader.biBitCount,
         bmi->bmiHeader.biHeight,
         bmi->bmiHeader.biWidth,
-        debugstr_an( (const char *)&bmi->bmiHeader.biCompression, 4 ) );
+        debugstr_fourcc(bmi->bmiHeader.biCompression));
 }
 
 static inline int ICCVID_CheckMask(RGBQUAD bmiColors[3], COLORREF redMask, COLORREF blueMask, COLORREF greenMask)
@@ -764,7 +758,7 @@ static inline int ICCVID_CheckMask(RGBQUAD bmiColors[3], COLORREF redMask, COLOR
     COLORREF realBlueMask = MAKECOLOUR32(bmiColors[1].rgbRed, bmiColors[1].rgbGreen, bmiColors[1].rgbBlue);
     COLORREF realGreenMask = MAKECOLOUR32(bmiColors[2].rgbRed, bmiColors[2].rgbGreen, bmiColors[2].rgbBlue);
 
-    TRACE("\nbmiColors[0] = 0x%08x\nbmiColors[1] = 0x%08x\nbmiColors[2] = 0x%08x\n",
+    TRACE("\nbmiColors[0] = 0x%08lx\nbmiColors[1] = 0x%08lx\nbmiColors[2] = 0x%08lx\n",
         realRedMask, realBlueMask, realGreenMask);
         
     if ((realRedMask == redMask) &&
@@ -803,26 +797,21 @@ static LRESULT ICCVID_DecompressQuery( ICCVID_Info *info, LPBITMAPINFO in, LPBIT
         if( in->bmiHeader.biWidth != out->bmiHeader.biWidth )
             return ICERR_BADFORMAT;
 
-        switch( out->bmiHeader.biBitCount )
+        switch( out->bmiHeader.biCompression )
         {
-        case 16:
-            if ( out->bmiHeader.biCompression == BI_BITFIELDS )
-            {
-                if ( !ICCVID_CheckMask(out->bmiColors, 0x7C00, 0x03E0, 0x001F) &&
-                     !ICCVID_CheckMask(out->bmiColors, 0xF800, 0x07E0, 0x001F) )
-                {
-                    TRACE("unsupported output bit field(s) for 16-bit colors\n");
-                    return ICERR_BADFORMAT;
-                }
-            }
+        case BI_RGB:
+            if ( out->bmiHeader.biBitCount == 16 || out->bmiHeader.biBitCount == 24 || out->bmiHeader.biBitCount == 32 )
+                return ICERR_OK;
             break;
-        case 24:
-        case 32:
+        case BI_BITFIELDS:
+            if ( out->bmiHeader.biBitCount == 16 && ICCVID_CheckMask(out->bmiColors, 0x7C00, 0x03E0, 0x001F) )
+                return ICERR_OK;
+            if ( out->bmiHeader.biBitCount == 16 && ICCVID_CheckMask(out->bmiColors, 0xF800, 0x07E0, 0x001F) )
+                return ICERR_OK;
             break;
-        default:
-            TRACE("unsupported output bitcount = %d\n", out->bmiHeader.biBitCount );
-            return ICERR_BADFORMAT;
         }
+        TRACE("unsupported output format\n");
+        return ICERR_BADFORMAT;
     }
 
     return ICERR_OK;
@@ -844,9 +833,9 @@ static LRESULT ICCVID_DecompressGetFormat( ICCVID_Info *info, LPBITMAPINFO in, L
     if( out )
     {
         memcpy( out, in, size );
+        out->bmiHeader.biBitCount = 24;
         out->bmiHeader.biCompression = BI_RGB;
-        out->bmiHeader.biSizeImage = in->bmiHeader.biHeight
-                                   * in->bmiHeader.biWidth *4;
+        out->bmiHeader.biSizeImage = get_stride(in->bmiHeader.biWidth, 24) * in->bmiHeader.biHeight;
         return ICERR_OK;
     }
     return size;
@@ -893,7 +882,7 @@ static LRESULT ICCVID_Decompress( ICCVID_Info *info, ICDECOMPRESS *icd, DWORD si
     LONG width, height;
     BOOL inverted;
 
-    TRACE("ICM_DECOMPRESS %p %p %d\n", info, icd, size);
+    TRACE("ICM_DECOMPRESS %p %p %ld\n", info, icd, size);
 
     if( (info==NULL) || (info->dwMagic!=ICCVID_MAGIC) )
         return ICERR_BADPARAM;
@@ -918,7 +907,7 @@ static LRESULT ICCVID_DecompressEx( ICCVID_Info *info, ICDECOMPRESSEX *icd, DWOR
     LONG width, height;
     BOOL inverted;
 
-    TRACE("ICM_DECOMPRESSEX %p %p %d\n", info, icd, size);
+    TRACE("ICM_DECOMPRESSEX %p %p %ld\n", info, icd, size);
 
     if( (info==NULL) || (info->dwMagic!=ICCVID_MAGIC) )
         return ICERR_BADPARAM;
@@ -946,7 +935,7 @@ static LRESULT ICCVID_Close( ICCVID_Info *info )
         return 0;
     if( info->cvinfo )
         free_cvinfo( info->cvinfo );
-    heap_free( info );
+    free( info );
     return 1;
 }
 
@@ -962,8 +951,8 @@ static LRESULT ICCVID_GetInfo( ICCVID_Info *info, ICINFO *icinfo, DWORD dwSize )
     icinfo->dwVersion = ICVERSION;
     icinfo->dwVersionICM = ICVERSION;
 
-    LoadStringW(ICCVID_hModule, IDS_NAME, icinfo->szName, sizeof(icinfo->szName)/sizeof(WCHAR));
-    LoadStringW(ICCVID_hModule, IDS_DESCRIPTION, icinfo->szDescription, sizeof(icinfo->szDescription)/sizeof(WCHAR));
+    LoadStringW(ICCVID_hModule, IDS_NAME, icinfo->szName, ARRAY_SIZE(icinfo->szName));
+    LoadStringW(ICCVID_hModule, IDS_DESCRIPTION, icinfo->szDescription, ARRAY_SIZE(icinfo->szDescription));
     /* msvfw32 will fill icinfo->szDriver for us */
 
     return sizeof(ICINFO);
@@ -984,7 +973,7 @@ LRESULT WINAPI ICCVID_DriverProc( DWORD_PTR dwDriverId, HDRVR hdrvr, UINT msg,
 {
     ICCVID_Info *info = (ICCVID_Info *) dwDriverId;
 
-    TRACE("%ld %p %d %ld %ld\n", dwDriverId, hdrvr, msg, lParam1, lParam2);
+    TRACE("%Id %p %d %Id %Id\n", dwDriverId, hdrvr, msg, lParam1, lParam2);
 
     switch( msg )
     {
@@ -1006,7 +995,7 @@ LRESULT WINAPI ICCVID_DriverProc( DWORD_PTR dwDriverId, HDRVR hdrvr, UINT msg,
 
         if (icinfo && compare_fourcc(icinfo->fccType, ICTYPE_VIDEO)) return 0;
 
-        info = heap_alloc( sizeof (ICCVID_Info) );
+        info = malloc(sizeof(ICCVID_Info));
         if( info )
         {
             info->dwMagic = ICCVID_MAGIC;
@@ -1048,14 +1037,14 @@ LRESULT WINAPI ICCVID_DriverProc( DWORD_PTR dwDriverId, HDRVR hdrvr, UINT msg,
         return ICERR_UNSUPPORTED;
 
     default:
-        FIXME("Unknown message: %04x %ld %ld\n", msg, lParam1, lParam2);
+        FIXME("Unknown message: %04x %Id %Id\n", msg, lParam1, lParam2);
     }
     return ICERR_UNSUPPORTED;
 }
 
 BOOL WINAPI DllMain(HINSTANCE hModule, DWORD dwReason, LPVOID lpReserved)
 {
-    TRACE("(%p,%d,%p)\n", hModule, dwReason, lpReserved);
+    TRACE("(%p,%ld,%p)\n", hModule, dwReason, lpReserved);
 
     switch (dwReason)
     {
