@@ -27,6 +27,8 @@
 #include "wingdi.h"
 
 #include "dinput.h"
+#include "dbt.h"
+#include "ddk/hidclass.h"
 
 #include "wine/debug.h"
 #include "wine/list.h"
@@ -64,6 +66,7 @@ static IDirectInputDevice8W *device_selected;
 
 static HWND dialog_hwnd;
 static HANDLE state_event;
+static HDEVNOTIFY devnotify;
 
 static BOOL CALLBACK enum_effects( const DIEFFECTINFOW *info, void *context )
 {
@@ -306,7 +309,7 @@ static DWORD WINAPI input_thread( void *param )
     return 0;
 }
 
-static void draw_axis_view( HDC hdc, RECT rect, const WCHAR *name, LONG value )
+static void draw_axis_view( HDC hdc, RECT rect, const WCHAR *name, double value )
 {
     POINT center =
     {
@@ -314,7 +317,7 @@ static void draw_axis_view( HDC hdc, RECT rect, const WCHAR *name, LONG value )
         .y = (rect.top + rect.bottom) / 2,
     };
     LONG w = (rect.bottom - rect.top + 1) / 3;
-    LONG x = rect.left + 20 + (w + 1) / 2 + MulDiv( value, rect.right - rect.left - 20 - w, 0xffff );
+    LONG x = rect.left + 20 + (w + 1) / 2 + value * (rect.right - rect.left - 20 - w);
     COLORREF color;
     HFONT font;
 
@@ -351,6 +354,53 @@ static void draw_axis_view( HDC hdc, RECT rect, const WCHAR *name, LONG value )
     }
 }
 
+void paint_axes_view( HWND hwnd, UINT32 count, double *axes, const WCHAR **names )
+{
+    RECT rect, tmp_rect;
+    PAINTSTRUCT paint;
+    HDC hdc;
+
+    hdc = BeginPaint( hwnd, &paint );
+
+    GetClientRect( hwnd, &rect );
+    rect.bottom = rect.top + (rect.bottom - rect.top - 2) / 4 - 2;
+    rect.right = rect.left + (rect.right - rect.left) / 2 - 10;
+
+    OffsetRect( &rect, 5, 2 );
+    draw_axis_view( hdc, rect, names[0], axes[0] );
+
+    tmp_rect = rect;
+    OffsetRect( &rect, rect.right - rect.left + 10, 0 );
+    draw_axis_view( hdc, rect, names[4], axes[4] );
+    rect = tmp_rect;
+
+    OffsetRect( &rect, 0, rect.bottom - rect.top + 2 );
+    draw_axis_view( hdc, rect, names[1], axes[1] );
+
+    tmp_rect = rect;
+    OffsetRect( &rect, rect.right - rect.left + 10, 0 );
+    draw_axis_view( hdc, rect, names[5], axes[5] );
+    rect = tmp_rect;
+
+    OffsetRect( &rect, 0, rect.bottom - rect.top + 2 );
+    draw_axis_view( hdc, rect, names[2], axes[2] );
+
+    tmp_rect = rect;
+    OffsetRect( &rect, rect.right - rect.left + 10, 0 );
+    draw_axis_view( hdc, rect, names[6], axes[6] );
+    rect = tmp_rect;
+
+    OffsetRect( &rect, 0, rect.bottom - rect.top + 2 );
+    draw_axis_view( hdc, rect, names[3], axes[3] );
+
+    tmp_rect = rect;
+    OffsetRect( &rect, rect.right - rect.left + 10, 0 );
+    draw_axis_view( hdc, rect, names[7], axes[7] );
+    rect = tmp_rect;
+
+    EndPaint( hwnd, &paint );
+}
+
 static void draw_pov_view( HDC hdc, RECT rect, DWORD value )
 {
     POINT points[] =
@@ -404,6 +454,30 @@ static void draw_pov_view( HDC hdc, RECT rect, DWORD value )
     if (value != -1) Polygon( hdc, points + value / 4500 * 2, 3 );
 }
 
+void paint_povs_view( HWND hwnd, UINT32 count, UINT32 *povs )
+{
+    PAINTSTRUCT paint;
+    RECT rect;
+    HDC hdc;
+
+    hdc = BeginPaint( hwnd, &paint );
+
+    GetClientRect( hwnd, &rect );
+    rect.bottom = rect.top + (rect.bottom - rect.top - 5) / 2 - 5;
+    rect.right = rect.left + (rect.bottom - rect.top);
+
+    OffsetRect( &rect, 5, 5 );
+    draw_pov_view( hdc, rect, povs[0] );
+    OffsetRect( &rect, rect.right - rect.left + 5, 0 );
+    draw_pov_view( hdc, rect, povs[1] );
+    OffsetRect( &rect, rect.left - rect.right - 5, rect.bottom - rect.top + 5 );
+    draw_pov_view( hdc, rect, povs[1] );
+    OffsetRect( &rect, rect.right - rect.left + 5, 0 );
+    draw_pov_view( hdc, rect, povs[2] );
+
+    EndPaint( hwnd, &paint );
+}
+
 static void draw_button_view( HDC hdc, RECT rect, BOOL set, const WCHAR *name )
 {
     COLORREF color;
@@ -429,18 +503,58 @@ static void draw_button_view( HDC hdc, RECT rect, BOOL set, const WCHAR *name )
     SelectObject( hdc, font );
 }
 
+void paint_buttons_view( HWND hwnd, UINT32 count, BYTE *buttons )
+{
+    UINT i, j, offs, size, step, space = 2;
+    PAINTSTRUCT paint;
+    RECT rect;
+    HDC hdc;
+
+    if (count <= 48) step = 16;
+    else step = 24;
+
+    hdc = BeginPaint( hwnd, &paint );
+
+    GetClientRect( hwnd, &rect );
+    FillRect( hdc, &rect, (HBRUSH)(COLOR_WINDOW + 1) );
+
+    size = (rect.right - rect.left - space) / step;
+    offs = (rect.right - rect.left - step * size - space) / 2;
+    OffsetRect( &rect, offs, offs );
+    rect.right = rect.left + size - space;
+    rect.bottom = rect.top + size - space;
+
+    for (i = 0; i < count;)
+    {
+        RECT first = rect;
+
+        for (j = 0; j < step && i < count; j++, i++)
+        {
+            WCHAR buffer[3];
+            if (step == 24) swprintf( buffer, ARRAY_SIZE(buffer), L"%02x", i );
+            else swprintf( buffer, ARRAY_SIZE(buffer), L"%d", i );
+            draw_button_view( hdc, rect, buttons[i], buffer );
+            OffsetRect( &rect, size, 0 );
+        }
+
+        rect = first;
+        OffsetRect( &rect, 0, size );
+    }
+
+    EndPaint( hwnd, &paint );
+}
+
 LRESULT CALLBACK test_di_axes_window_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
 {
     TRACE( "hwnd %p, msg %#x, wparam %#Ix, lparam %#Ix\n", hwnd, msg, wparam, lparam );
 
     if (msg == WM_PAINT)
     {
+        static const WCHAR *names[] = { L"X", L"Y", L"Z", L"S", L"Rx", L"Ry", L"Rz", L"Rs" };
         DIDEVCAPS caps = {.dwSize = sizeof(DIDEVCAPS)};
+        double axes[16], *axis = axes;
         IDirectInputDevice8W *device;
         DIJOYSTATE2 state = {0};
-        RECT rect, tmp_rect;
-        PAINTSTRUCT paint;
-        HDC hdc;
 
         if ((device = get_selected_device()))
         {
@@ -449,46 +563,16 @@ LRESULT CALLBACK test_di_axes_window_proc( HWND hwnd, UINT msg, WPARAM wparam, L
             IDirectInputDevice8_Release( device );
         }
 
-        hdc = BeginPaint( hwnd, &paint );
-
-        GetClientRect( hwnd, &rect );
-        rect.bottom = rect.top + (rect.bottom - rect.top - 2) / 4 - 2;
-        rect.right = rect.left + (rect.right - rect.left) / 2 - 10;
-
-        OffsetRect( &rect, 5, 2 );
-        draw_axis_view( hdc, rect, L"X", state.lX );
-
-        tmp_rect = rect;
-        OffsetRect( &rect, rect.right - rect.left + 10, 0 );
-        draw_axis_view( hdc, rect, L"Rx", state.lRx );
-        rect = tmp_rect;
-
-        OffsetRect( &rect, 0, rect.bottom - rect.top + 2 );
-        draw_axis_view( hdc, rect, L"Y", state.lY );
-
-        tmp_rect = rect;
-        OffsetRect( &rect, rect.right - rect.left + 10, 0 );
-        draw_axis_view( hdc, rect, L"Ry", state.lRy );
-        rect = tmp_rect;
-
-        OffsetRect( &rect, 0, rect.bottom - rect.top + 2 );
-        draw_axis_view( hdc, rect, L"Z", state.lZ );
-
-        tmp_rect = rect;
-        OffsetRect( &rect, rect.right - rect.left + 10, 0 );
-        draw_axis_view( hdc, rect, L"Rz", state.lRz );
-        rect = tmp_rect;
-
-        OffsetRect( &rect, 0, rect.bottom - rect.top + 2 );
-        draw_axis_view( hdc, rect, L"S", state.rglSlider[0] );
-
-        tmp_rect = rect;
-        OffsetRect( &rect, rect.right - rect.left + 10, 0 );
-        draw_axis_view( hdc, rect, L"Rs", state.rglSlider[1] );
-        rect = tmp_rect;
-
-        EndPaint( hwnd, &paint );
+        *axis++ = (double)state.lX / 65535.0;
+        *axis++ = (double)state.lY / 65535.0;
+        *axis++ = (double)state.lZ / 65535.0;
+        *axis++ = (double)state.rglSlider[0] / 65535.0;
+        *axis++ = (double)state.lRx / 65535.0;
+        *axis++ = (double)state.lRy / 65535.0;
+        *axis++ = (double)state.lRz / 65535.0;
+        *axis++ = (double)state.rglSlider[1] / 65535.0;
 
+        paint_axes_view( hwnd, min( caps.dwAxes, ARRAY_SIZE(axes) ), axes, names );
         return 0;
     }
 
@@ -504,9 +588,6 @@ LRESULT CALLBACK test_di_povs_window_proc( HWND hwnd, UINT msg, WPARAM wparam, L
         DIDEVCAPS caps = {.dwSize = sizeof(DIDEVCAPS)};
         IDirectInputDevice8W *device;
         DIJOYSTATE2 state = {0};
-        PAINTSTRUCT paint;
-        RECT rect;
-        HDC hdc;
 
         if ((device = get_selected_device()))
         {
@@ -515,23 +596,7 @@ LRESULT CALLBACK test_di_povs_window_proc( HWND hwnd, UINT msg, WPARAM wparam, L
             IDirectInputDevice8_Release( device );
         }
 
-        hdc = BeginPaint( hwnd, &paint );
-
-        GetClientRect( hwnd, &rect );
-        rect.bottom = rect.top + (rect.bottom - rect.top - 5) / 2 - 5;
-        rect.right = rect.left + (rect.bottom - rect.top);
-
-        OffsetRect( &rect, 5, 5 );
-        draw_pov_view( hdc, rect, state.rgdwPOV[0] );
-        OffsetRect( &rect, rect.right - rect.left + 5, 0 );
-        draw_pov_view( hdc, rect, state.rgdwPOV[1] );
-        OffsetRect( &rect, rect.left - rect.right - 5, rect.bottom - rect.top + 5 );
-        draw_pov_view( hdc, rect, state.rgdwPOV[1] );
-        OffsetRect( &rect, rect.right - rect.left + 5, 0 );
-        draw_pov_view( hdc, rect, state.rgdwPOV[2] );
-
-        EndPaint( hwnd, &paint );
-
+        paint_povs_view( hwnd, caps.dwPOVs, (UINT32 *)state.rgdwPOV );
         return 0;
     }
 
@@ -545,12 +610,8 @@ LRESULT CALLBACK test_di_buttons_window_proc( HWND hwnd, UINT msg, WPARAM wparam
     if (msg == WM_PAINT)
     {
         DIDEVCAPS caps = {.dwSize = sizeof(DIDEVCAPS)};
-        UINT i, j, offs, size, step, space = 2;
         IDirectInputDevice8W *device;
         DIJOYSTATE2 state = {0};
-        PAINTSTRUCT paint;
-        RECT rect;
-        HDC hdc;
 
         if ((device = get_selected_device()))
         {
@@ -559,39 +620,7 @@ LRESULT CALLBACK test_di_buttons_window_proc( HWND hwnd, UINT msg, WPARAM wparam
             IDirectInputDevice8_Release( device );
         }
 
-        if (caps.dwButtons <= 48) step = 16;
-        else step = 24;
-
-        hdc = BeginPaint( hwnd, &paint );
-
-        GetClientRect( hwnd, &rect );
-        FillRect( hdc, &rect, (HBRUSH)(COLOR_WINDOW + 1) );
-
-        size = (rect.right - rect.left - space) / step;
-        offs = (rect.right - rect.left - step * size - space) / 2;
-        OffsetRect( &rect, offs, offs );
-        rect.right = rect.left + size - space;
-        rect.bottom = rect.top + size - space;
-
-        for (i = 0; i < ARRAY_SIZE(state.rgbButtons) && i < caps.dwButtons;)
-        {
-            RECT first = rect;
-
-            for (j = 0; j < step && i < caps.dwButtons; j++, i++)
-            {
-                WCHAR buffer[3];
-                if (step == 24) swprintf( buffer, ARRAY_SIZE(buffer), L"%02x", i );
-                else swprintf( buffer, ARRAY_SIZE(buffer), L"%d", i );
-                draw_button_view( hdc, rect, state.rgbButtons[i], buffer );
-                OffsetRect( &rect, size, 0 );
-            }
-
-            rect = first;
-            OffsetRect( &rect, 0, size );
-        }
-
-        EndPaint( hwnd, &paint );
-
+        paint_buttons_view( hwnd, min( caps.dwButtons, ARRAY_SIZE(state.rgbButtons) ), state.rgbButtons );
         return 0;
     }
 
@@ -748,6 +777,12 @@ static void update_device_views( HWND hwnd )
 INT_PTR CALLBACK test_di_dialog_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
 {
     static HANDLE thread, thread_stop;
+    DEV_BROADCAST_DEVICEINTERFACE_W filter =
+    {
+        .dbcc_size = sizeof(DEV_BROADCAST_DEVICEINTERFACE_W),
+        .dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE,
+        .dbcc_classguid = GUID_DEVINTERFACE_HID,
+    };
 
     TRACE( "hwnd %p, msg %#x, wparam %#Ix, lparam %#Ix\n", hwnd, msg, wparam, lparam );
 
@@ -783,6 +818,7 @@ INT_PTR CALLBACK test_di_dialog_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM
             state_event = CreateEventW( NULL, FALSE, FALSE, NULL );
             thread_stop = CreateEventW( NULL, FALSE, FALSE, NULL );
 
+            devnotify = RegisterDeviceNotificationW( hwnd, &filter, DEVICE_NOTIFY_WINDOW_HANDLE );
             update_di_devices( hwnd );
 
             SendDlgItemMessageW( hwnd, IDC_DI_DEVICES, CB_SETCURSEL, 0, 0 );
@@ -796,6 +832,9 @@ INT_PTR CALLBACK test_di_dialog_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM
 
         case PSN_RESET:
         case PSN_KILLACTIVE:
+            if (!devnotify) break;
+            UnregisterDeviceNotification( devnotify );
+            devnotify = NULL;
             SetEvent( thread_stop );
             /* wait for the input thread to stop, processing any WM_USER message from it */
             while (MsgWaitForMultipleObjects( 1, &thread, FALSE, INFINITE, QS_ALLINPUT ) == 1)
@@ -817,6 +856,18 @@ INT_PTR CALLBACK test_di_dialog_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM
         }
         return TRUE;
 
+    case WM_DEVICECHANGE:
+        update_di_devices( hwnd );
+
+        if (SendDlgItemMessageW( hwnd, IDC_DI_DEVICES, CB_GETCURSEL, 0, 0 ) >= 0) return TRUE;
+
+        SendDlgItemMessageW( hwnd, IDC_DI_DEVICES, CB_SETCURSEL, 0, 0 );
+        handle_di_devices_change( hwnd );
+
+        SendDlgItemMessageW( hwnd, IDC_DI_EFFECTS, LB_SETCURSEL, 0, 0 );
+        handle_di_effects_change( hwnd );
+        return TRUE;
+
     case WM_USER:
         update_device_views( hwnd );
         return TRUE;
