@@ -769,25 +769,23 @@ static VOID test_thread_priority(void)
                        0,curthreadId);
      ok(access_thread!=NULL,"OpenThread returned an invalid handle\n");
      if (access_thread!=NULL) {
-       todo_wine obey_ar(pSetThreadPriorityBoost(access_thread,1)==0);
-       todo_wine obey_ar(pGetThreadPriorityBoost(access_thread,&disabled)==0);
+       obey_ar(pSetThreadPriorityBoost(access_thread, 1) == 0);
+       obey_ar(pGetThreadPriorityBoost(access_thread, &disabled) == 0);
        ok(CloseHandle(access_thread),"Error Closing thread handle\n");
      }
    }
 
-   rc = pSetThreadPriorityBoost(curthread,1);
-   ok( rc != 0, "error=%ld\n",GetLastError());
-   todo_wine {
-     rc=pGetThreadPriorityBoost(curthread,&disabled);
-     ok(rc!=0 && disabled==1,
-        "rc=%d error=%ld disabled=%d\n",rc,GetLastError(),disabled);
-   }
+   rc = pSetThreadPriorityBoost(curthread, 1);
+   ok(rc != 0, "error=%ld\n", GetLastError());
+   rc = pGetThreadPriorityBoost(curthread, &disabled);
+   ok(rc != 0 && disabled == 1,
+      "rc=%d error=%ld disabled=%d\n", rc, GetLastError(), disabled);
 
-   rc = pSetThreadPriorityBoost(curthread,0);
-   ok( rc != 0, "error=%ld\n",GetLastError());
-   rc=pGetThreadPriorityBoost(curthread,&disabled);
-   ok(rc!=0 && disabled==0,
-      "rc=%d error=%ld disabled=%d\n",rc,GetLastError(),disabled);
+   rc = pSetThreadPriorityBoost(curthread, 0);
+   ok(rc != 0, "error=%ld\n", GetLastError());
+   rc = pGetThreadPriorityBoost(curthread, &disabled);
+   ok(rc != 0 && disabled == 0,
+      "rc=%d error=%ld disabled=%d\n", rc, GetLastError(), disabled);
 }
 
 /* check the GetThreadTimes function */
@@ -1889,7 +1887,13 @@ struct fpu_thread_ctx
 
 static inline unsigned long get_fpu_cw(void)
 {
-#if defined(__i386__) || defined(__x86_64__)
+#ifdef __arm64ec__
+    extern NTSTATUS (*__os_arm64x_get_x64_information)(ULONG,void*,void*);
+    unsigned int cw, sse;
+    __os_arm64x_get_x64_information( 0, &sse, NULL );
+    __os_arm64x_get_x64_information( 2, &cw, NULL );
+    return MAKELONG( cw, sse );
+#elif defined(__i386__) || defined(__x86_64__)
     WORD cw = 0;
     unsigned int sse = 0;
 #ifdef _MSC_VER
@@ -1928,7 +1932,7 @@ static inline void fpu_invalid_operation(void)
 
     d = acos(2.0);
     ok(_isnan(d), "d = %lf\n", d);
-    ok(_statusfp() == _SW_INVALID, "_statusfp() = %x\n", _statusfp());
+    ok(_statusfp() & _SW_INVALID, "_statusfp() = %x\n", _statusfp());
 }
 
 static DWORD WINAPI fpu_thread(void *param)
