@@ -30,61 +30,52 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
 
-static BOOL load_fusion_dlls( MSIPACKAGE *package )
+static void load_fusion_dlls( MSIPACKAGE *package )
 {
-    static const WCHAR szFusion[]    = {'f','u','s','i','o','n','.','d','l','l',0};
-    static const WCHAR szMscoree[]   = {'\\','m','s','c','o','r','e','e','.','d','l','l',0};
-    static const WCHAR szVersion10[] = {'v','1','.','0','.','3','7','0','5',0};
-    static const WCHAR szVersion11[] = {'v','1','.','1','.','4','3','2','2',0};
-    static const WCHAR szVersion20[] = {'v','2','.','0','.','5','0','7','2','7',0};
-    static const WCHAR szVersion40[] = {'v','4','.','0','.','3','0','3','1','9',0};
     HRESULT (WINAPI *pLoadLibraryShim)( const WCHAR *, const WCHAR *, void *, HMODULE * );
     WCHAR path[MAX_PATH];
-    DWORD len = GetSystemDirectoryW( path, MAX_PATH );
 
-    lstrcpyW( path + len, szMscoree );
-    if (package->hmscoree || !(package->hmscoree = LoadLibraryW( path ))) return TRUE;
+    wcscpy(path, sysdir);
+    lstrcpyW( path + sysdir_len, L"\\mscoree.dll" );
+    if (!package->hmscoree && !(package->hmscoree = LoadLibraryW( path ))) return;
     if (!(pLoadLibraryShim = (void *)GetProcAddress( package->hmscoree, "LoadLibraryShim" )))
     {
         FreeLibrary( package->hmscoree );
         package->hmscoree = NULL;
-        return TRUE;
+        return;
     }
 
-    pLoadLibraryShim( szFusion, szVersion10, NULL, &package->hfusion10 );
-    pLoadLibraryShim( szFusion, szVersion11, NULL, &package->hfusion11 );
-    pLoadLibraryShim( szFusion, szVersion20, NULL, &package->hfusion20 );
-    pLoadLibraryShim( szFusion, szVersion40, NULL, &package->hfusion40 );
-
-    return TRUE;
+    if (!package->hfusion10) pLoadLibraryShim( L"fusion.dll", L"v1.0.3705", NULL, &package->hfusion10 );
+    if (!package->hfusion11) pLoadLibraryShim( L"fusion.dll", L"v1.1.4322", NULL, &package->hfusion11 );
+    if (!package->hfusion20) pLoadLibraryShim( L"fusion.dll", L"v2.0.50727", NULL, &package->hfusion20 );
+    if (!package->hfusion40) pLoadLibraryShim( L"fusion.dll", L"v4.0.30319", NULL, &package->hfusion40 );
 }
 
-BOOL msi_init_assembly_caches( MSIPACKAGE *package )
+static BOOL init_assembly_caches( MSIPACKAGE *package )
 {
     HRESULT (WINAPI *pCreateAssemblyCache)( IAssemblyCache **, DWORD );
 
-    if (package->cache_sxs) return TRUE;
-    if (CreateAssemblyCache( &package->cache_sxs, 0 ) != S_OK) return FALSE;
+    if (!package->cache_sxs && CreateAssemblyCache( &package->cache_sxs, 0 ) != S_OK) return FALSE;
 
-    if (!load_fusion_dlls( package )) return FALSE;
+    load_fusion_dlls( package );
     package->pGetFileVersion = (void *)GetProcAddress( package->hmscoree, "GetFileVersion" ); /* missing from v1.0.3705 */
 
-    if (package->hfusion10)
+    if (package->hfusion10 && !package->cache_net[CLR_VERSION_V10])
     {
         pCreateAssemblyCache = (void *)GetProcAddress( package->hfusion10, "CreateAssemblyCache" );
         pCreateAssemblyCache( &package->cache_net[CLR_VERSION_V10], 0 );
     }
-    if (package->hfusion11)
+    if (package->hfusion11 && !package->cache_net[CLR_VERSION_V11])
     {
         pCreateAssemblyCache = (void *)GetProcAddress( package->hfusion11, "CreateAssemblyCache" );
         pCreateAssemblyCache( &package->cache_net[CLR_VERSION_V11], 0 );
     }
-    if (package->hfusion20)
+    if (package->hfusion20 && !package->cache_net[CLR_VERSION_V20])
     {
         pCreateAssemblyCache = (void *)GetProcAddress( package->hfusion20, "CreateAssemblyCache" );
         pCreateAssemblyCache( &package->cache_net[CLR_VERSION_V20], 0 );
     }
-    if (package->hfusion40)
+    if (package->hfusion40 && !package->cache_net[CLR_VERSION_V40])
     {
         pCreateAssemblyCache = (void *)GetProcAddress( package->hfusion40, "CreateAssemblyCache" );
         pCreateAssemblyCache( &package->cache_net[CLR_VERSION_V40], 0 );
@@ -129,16 +120,11 @@ void msi_destroy_assembly_caches( MSIPACKAGE *package )
 
 static MSIRECORD *get_assembly_record( MSIPACKAGE *package, const WCHAR *comp )
 {
-    static const WCHAR query[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-         '`','M','s','i','A','s','s','e','m','b','l','y','`',' ',
-         'W','H','E','R','E',' ','`','C','o','m','p','o','n','e','n','t','_','`',
-         ' ','=',' ','\'','%','s','\'',0};
     MSIQUERY *view;
     MSIRECORD *rec;
     UINT r;
 
-    r = MSI_OpenQuery( package->db, &view, query, comp );
+    r = MSI_OpenQuery( package->db, &view, L"SELECT * FROM `MsiAssembly` WHERE `Component_` = '%s'", comp );
     if (r != ERROR_SUCCESS)
         return NULL;
 
@@ -163,43 +149,35 @@ static MSIRECORD *get_assembly_record( MSIPACKAGE *package, const WCHAR *comp )
 
 struct assembly_name
 {
-    UINT    count;
+    DWORD   count;
     UINT    index;
     WCHAR **attrs;
 };
 
 static UINT get_assembly_name_attribute( MSIRECORD *rec, LPVOID param )
 {
-    static const WCHAR fmtW[] = {'%','s','=','"','%','s','"',0};
-    static const WCHAR nameW[] = {'n','a','m','e',0};
     struct assembly_name *name = param;
     const WCHAR *attr = MSI_RecordGetString( rec, 2 );
     const WCHAR *value = MSI_RecordGetString( rec, 3 );
-    int len = lstrlenW( fmtW ) + lstrlenW( attr ) + lstrlenW( value );
+    int len = lstrlenW( L"%s=\"%s\"" ) + lstrlenW( attr ) + lstrlenW( value );
 
-    if (!(name->attrs[name->index] = msi_alloc( len * sizeof(WCHAR) )))
+    if (!(name->attrs[name->index] = malloc( len * sizeof(WCHAR) )))
         return ERROR_OUTOFMEMORY;
 
-    if (!wcsicmp( attr, nameW )) lstrcpyW( name->attrs[name->index++], value );
-    else swprintf( name->attrs[name->index++], len, fmtW, attr, value );
+    if (!wcsicmp( attr, L"name" )) lstrcpyW( name->attrs[name->index++], value );
+    else swprintf( name->attrs[name->index++], len, L"%s=\"%s\"", attr, value );
     return ERROR_SUCCESS;
 }
 
 static WCHAR *get_assembly_display_name( MSIDATABASE *db, const WCHAR *comp, MSIASSEMBLY *assembly )
 {
-    static const WCHAR commaW[] = {',',0};
-    static const WCHAR queryW[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        '`','M','s','i','A','s','s','e','m','b','l','y','N','a','m','e','`',' ',
-        'W','H','E','R','E',' ','`','C','o','m','p','o','n','e','n','t','_','`',
-        ' ','=',' ','\'','%','s','\'',0};
     struct assembly_name name;
     WCHAR *display_name = NULL;
     MSIQUERY *view;
     UINT i, r;
     int len;
 
-    r = MSI_OpenQuery( db, &view, queryW, comp );
+    r = MSI_OpenQuery( db, &view, L"SELECT * FROM `MsiAssemblyName` WHERE `Component_` = '%s'", comp );
     if (r != ERROR_SUCCESS)
         return NULL;
 
@@ -209,7 +187,7 @@ static WCHAR *get_assembly_display_name( MSIDATABASE *db, const WCHAR *comp, MSI
     MSI_IterateRecords( view, &name.count, NULL, NULL );
     if (!name.count) goto done;
 
-    name.attrs = msi_alloc( name.count * sizeof(WCHAR *) );
+    name.attrs = malloc( name.count * sizeof(WCHAR *) );
     if (!name.attrs) goto done;
 
     MSI_IterateRecords( view, NULL, get_assembly_name_attribute, &name );
@@ -217,14 +195,14 @@ static WCHAR *get_assembly_display_name( MSIDATABASE *db, const WCHAR *comp, MSI
     len = 0;
     for (i = 0; i < name.count; i++) len += lstrlenW( name.attrs[i] ) + 1;
 
-    display_name = msi_alloc( (len + 1) * sizeof(WCHAR) );
+    display_name = malloc( (len + 1) * sizeof(WCHAR) );
     if (display_name)
     {
         display_name[0] = 0;
         for (i = 0; i < name.count; i++)
         {
             lstrcatW( display_name, name.attrs[i] );
-            if (i < name.count - 1) lstrcatW( display_name, commaW );
+            if (i < name.count - 1) lstrcatW( display_name, L"," );
         }
     }
 
@@ -232,49 +210,31 @@ done:
     msiobj_release( &view->hdr );
     if (name.attrs)
     {
-        for (i = 0; i < name.count; i++) msi_free( name.attrs[i] );
-        msi_free( name.attrs );
+        for (i = 0; i < name.count; i++) free( name.attrs[i] );
+        free( name.attrs );
     }
     return display_name;
 }
 
-static BOOL is_assembly_installed( IAssemblyCache *cache, const WCHAR *display_name )
-{
-    HRESULT hr;
-    ASSEMBLY_INFO info;
-
-    if (!cache) return FALSE;
-
-    memset( &info, 0, sizeof(info) );
-    info.cbAssemblyInfo = sizeof(info);
-    hr = IAssemblyCache_QueryAssemblyInfo( cache, 0, display_name, &info );
-    if (hr == S_OK /* sxs version */ || hr == E_NOT_SUFFICIENT_BUFFER)
-    {
-        return (info.dwAssemblyFlags == ASSEMBLYINFO_FLAG_INSTALLED);
-    }
-    TRACE("QueryAssemblyInfo returned 0x%08x\n", hr);
-    return FALSE;
-}
-
 WCHAR *msi_get_assembly_path( MSIPACKAGE *package, const WCHAR *displayname )
 {
     HRESULT hr;
     ASSEMBLY_INFO info;
-    IAssemblyCache *cache = package->cache_net[CLR_VERSION_V40];
+    IAssemblyCache *cache;
 
-    if (!cache) return NULL;
+    if (!init_assembly_caches( package ) || !(cache = package->cache_net[CLR_VERSION_V40])) return NULL;
 
     memset( &info, 0, sizeof(info) );
     info.cbAssemblyInfo = sizeof(info);
     hr = IAssemblyCache_QueryAssemblyInfo( cache, 0, displayname, &info );
     if (hr != E_NOT_SUFFICIENT_BUFFER) return NULL;
 
-    if (!(info.pszCurrentAssemblyPathBuf = msi_alloc( info.cchBuf * sizeof(WCHAR) ))) return NULL;
+    if (!(info.pszCurrentAssemblyPathBuf = malloc( info.cchBuf * sizeof(WCHAR) ))) return NULL;
 
     hr = IAssemblyCache_QueryAssemblyInfo( cache, 0, displayname, &info );
     if (FAILED( hr ))
     {
-        msi_free( info.pszCurrentAssemblyPathBuf );
+        free( info.pszCurrentAssemblyPathBuf );
         return NULL;
     }
     TRACE("returning %s\n", debugstr_w(info.pszCurrentAssemblyPathBuf));
@@ -287,15 +247,16 @@ IAssemblyEnum *msi_create_assembly_enum( MSIPACKAGE *package, const WCHAR *displ
     IAssemblyName *name;
     IAssemblyEnum *ret;
     WCHAR *str;
-    UINT len = 0;
+    DWORD len = 0;
 
-    if (!package->pCreateAssemblyNameObject || !package->pCreateAssemblyEnum) return NULL;
+    if (!init_assembly_caches( package ) || !package->pCreateAssemblyNameObject || !package->pCreateAssemblyEnum)
+        return NULL;
 
     hr = package->pCreateAssemblyNameObject( &name, displayname, CANOF_PARSE_DISPLAY_NAME, NULL );
     if (FAILED( hr )) return NULL;
 
     hr = IAssemblyName_GetName( name, &len, NULL );
-    if (hr != E_NOT_SUFFICIENT_BUFFER || !(str = msi_alloc( len * sizeof(WCHAR) )))
+    if (hr != E_NOT_SUFFICIENT_BUFFER || !(str = malloc( len * sizeof(WCHAR) )))
     {
         IAssemblyName_Release( name );
         return NULL;
@@ -305,12 +266,12 @@ IAssemblyEnum *msi_create_assembly_enum( MSIPACKAGE *package, const WCHAR *displ
     IAssemblyName_Release( name );
     if (FAILED( hr ))
     {
-        msi_free( str );
+        free( str );
         return NULL;
     }
 
     hr = package->pCreateAssemblyNameObject( &name, str, 0, NULL );
-    msi_free( str );
+    free( str );
     if (FAILED( hr )) return NULL;
 
     hr = package->pCreateAssemblyEnum( &ret, NULL, name, ASM_CACHE_GAC, NULL );
@@ -320,90 +281,49 @@ IAssemblyEnum *msi_create_assembly_enum( MSIPACKAGE *package, const WCHAR *displ
     return ret;
 }
 
-static const WCHAR clr_version_v10[] = {'v','1','.','0','.','3','7','0','5',0};
-static const WCHAR clr_version_v11[] = {'v','1','.','1','.','4','3','2','2',0};
-static const WCHAR clr_version_v20[] = {'v','2','.','0','.','5','0','7','2','7',0};
-static const WCHAR clr_version_v40[] = {'v','4','.','0','.','3','0','3','1','9',0};
-static const WCHAR clr_version_unknown[] = {'u','n','k','n','o','w','n',0};
-
 static const WCHAR *clr_version[] =
 {
-    clr_version_v10,
-    clr_version_v11,
-    clr_version_v20,
-    clr_version_v40
+    L"v1.0.3705",
+    L"v1.2.4322",
+    L"v2.0.50727",
+    L"v4.0.30319"
 };
 
-static const WCHAR *get_clr_version_str( enum clr_version version )
-{
-    if (version >= ARRAY_SIZE( clr_version )) return clr_version_unknown;
-    return clr_version[version];
-}
-
-/* assembly caches must be initialized */
 MSIASSEMBLY *msi_load_assembly( MSIPACKAGE *package, MSICOMPONENT *comp )
 {
     MSIRECORD *rec;
     MSIASSEMBLY *a;
 
     if (!(rec = get_assembly_record( package, comp->Component ))) return NULL;
-    if (!(a = msi_alloc_zero( sizeof(MSIASSEMBLY) )))
+    if (!(a = calloc( 1, sizeof(MSIASSEMBLY) )))
     {
         msiobj_release( &rec->hdr );
         return NULL;
     }
-    a->feature = strdupW( MSI_RecordGetString( rec, 2 ) );
+    a->feature = wcsdup( MSI_RecordGetString( rec, 2 ) );
     TRACE("feature %s\n", debugstr_w(a->feature));
 
-    a->manifest = strdupW( MSI_RecordGetString( rec, 3 ) );
+    a->manifest = wcsdup( MSI_RecordGetString( rec, 3 ) );
     TRACE("manifest %s\n", debugstr_w(a->manifest));
 
-    a->application = strdupW( MSI_RecordGetString( rec, 4 ) );
+    a->application = wcsdup( MSI_RecordGetString( rec, 4 ) );
     TRACE("application %s\n", debugstr_w(a->application));
 
     a->attributes = MSI_RecordGetInteger( rec, 5 );
-    TRACE("attributes %u\n", a->attributes);
+    TRACE( "attributes %lu\n", a->attributes );
 
     if (!(a->display_name = get_assembly_display_name( package->db, comp->Component, a )))
     {
         WARN("can't get display name\n");
         msiobj_release( &rec->hdr );
-        msi_free( a->feature );
-        msi_free( a->manifest );
-        msi_free( a->application );
-        msi_free( a );
+        free( a->feature );
+        free( a->manifest );
+        free( a->application );
+        free( a );
         return NULL;
     }
     TRACE("display name %s\n", debugstr_w(a->display_name));
 
-    if (a->application)
-    {
-        /* We can't check the manifest here because the target path may still change.
-           So we assume that the assembly is not installed and lean on the InstallFiles
-           action to determine which files need to be installed.
-         */
-        a->installed = FALSE;
-    }
-    else
-    {
-        if (a->attributes == msidbAssemblyAttributesWin32)
-            a->installed = is_assembly_installed( package->cache_sxs, a->display_name );
-        else
-        {
-            UINT i;
-            for (i = 0; i < CLR_VERSION_MAX; i++)
-            {
-                a->clr_version[i] = is_assembly_installed( package->cache_net[i], a->display_name );
-                if (a->clr_version[i])
-                {
-                    TRACE("runtime version %s\n", debugstr_w(get_clr_version_str( i )));
-                    a->installed = TRUE;
-                    break;
-                }
-            }
-        }
-    }
-    TRACE("assembly is %s\n", a->installed ? "installed" : "not installed");
     msiobj_release( &rec->hdr );
     return a;
 }
@@ -419,7 +339,7 @@ static enum clr_version get_clr_version( MSIPACKAGE *package, const WCHAR *filen
 
     hr = package->pGetFileVersion( filename, NULL, 0, &len );
     if (hr != E_NOT_SUFFICIENT_BUFFER) return CLR_VERSION_V11;
-    if ((strW = msi_alloc( len * sizeof(WCHAR) )))
+    if ((strW = malloc( len * sizeof(WCHAR) )))
     {
         hr = package->pGetFileVersion( filename, strW, len, &len );
         if (hr == S_OK)
@@ -428,7 +348,7 @@ static enum clr_version get_clr_version( MSIPACKAGE *package, const WCHAR *filen
             for (i = 0; i < CLR_VERSION_MAX; i++)
                 if (!wcscmp( strW, clr_version[i] )) version = i;
         }
-        msi_free( strW );
+        free( strW );
     }
     return version;
 }
@@ -441,6 +361,8 @@ UINT msi_install_assembly( MSIPACKAGE *package, MSICOMPONENT *comp )
     MSIASSEMBLY *assembly = comp->assembly;
     MSIFEATURE *feature = NULL;
 
+    if (!init_assembly_caches( package )) return ERROR_FUNCTION_FAILED;
+
     if (comp->assembly->feature)
         feature = msi_get_loaded_feature( package, comp->assembly->feature );
 
@@ -470,11 +392,10 @@ UINT msi_install_assembly( MSIPACKAGE *package, MSICOMPONENT *comp )
     hr = IAssemblyCache_InstallAssembly( cache, 0, manifest, NULL );
     if (hr != S_OK)
     {
-        ERR("Failed to install assembly %s (0x%08x)\n", debugstr_w(manifest), hr);
+        ERR( "failed to install assembly %s (%#lx)\n", debugstr_w(manifest), hr );
         return ERROR_FUNCTION_FAILED;
     }
     if (feature) feature->Action = INSTALLSTATE_LOCAL;
-    assembly->installed = TRUE;
     return ERROR_SUCCESS;
 }
 
@@ -485,6 +406,8 @@ UINT msi_uninstall_assembly( MSIPACKAGE *package, MSICOMPONENT *comp )
     MSIASSEMBLY *assembly = comp->assembly;
     MSIFEATURE *feature = NULL;
 
+    if (!init_assembly_caches( package )) return ERROR_FUNCTION_FAILED;
+
     if (comp->assembly->feature)
         feature = msi_get_loaded_feature( package, comp->assembly->feature );
 
@@ -499,7 +422,7 @@ UINT msi_uninstall_assembly( MSIPACKAGE *package, MSICOMPONENT *comp )
     {
         cache = package->cache_sxs;
         hr = IAssemblyCache_UninstallAssembly( cache, 0, assembly->display_name, NULL, NULL );
-        if (FAILED( hr )) WARN("failed to uninstall assembly 0x%08x\n", hr);
+        if (FAILED( hr )) WARN( "failed to uninstall assembly %#lx\n", hr );
     }
     else
     {
@@ -511,12 +434,11 @@ UINT msi_uninstall_assembly( MSIPACKAGE *package, MSICOMPONENT *comp )
             if (cache)
             {
                 hr = IAssemblyCache_UninstallAssembly( cache, 0, assembly->display_name, NULL, NULL );
-                if (FAILED( hr )) WARN("failed to uninstall assembly 0x%08x\n", hr);
+                if (FAILED( hr )) WARN( "failed to uninstall assembly %#lx\n", hr );
             }
         }
     }
     if (feature) feature->Action = INSTALLSTATE_ABSENT;
-    assembly->installed = FALSE;
     return ERROR_SUCCESS;
 }
 
@@ -525,7 +447,7 @@ static WCHAR *build_local_assembly_path( const WCHAR *filename )
     UINT i;
     WCHAR *ret;
 
-    if (!(ret = msi_alloc( (lstrlenW( filename ) + 1) * sizeof(WCHAR) )))
+    if (!(ret = malloc( (wcslen( filename ) + 1) * sizeof(WCHAR) )))
         return NULL;
 
     for (i = 0; filename[i]; i++)
@@ -539,30 +461,20 @@ static WCHAR *build_local_assembly_path( const WCHAR *filename )
 
 static LONG open_assemblies_key( UINT context, BOOL win32, HKEY *hkey )
 {
-    static const WCHAR path_win32[] =
-        {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
-          'I','n','s','t','a','l','l','e','r','\\','W','i','n','3','2','A','s','s','e','m','b','l','i','e','s','\\',0};
-    static const WCHAR path_dotnet[] =
-        {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
-         'I','n','s','t','a','l','l','e','r','\\','A','s','s','e','m','b','l','i','e','s','\\',0};
-    static const WCHAR classes_path_win32[] =
-        {'I','n','s','t','a','l','l','e','r','\\','W','i','n','3','2','A','s','s','e','m','b','l','i','e','s','\\',0};
-    static const WCHAR classes_path_dotnet[] =
-        {'I','n','s','t','a','l','l','e','r','\\','A','s','s','e','m','b','l','i','e','s','\\',0};
     HKEY root;
     const WCHAR *path;
 
     if (context == MSIINSTALLCONTEXT_MACHINE)
     {
         root = HKEY_CLASSES_ROOT;
-        if (win32) path = classes_path_win32;
-        else path = classes_path_dotnet;
+        if (win32) path = L"Installer\\Win32Assemblies\\";
+        else path = L"Installer\\Assemblies\\";
     }
     else
     {
         root = HKEY_CURRENT_USER;
-        if (win32) path = path_win32;
-        else path = path_dotnet;
+        if (win32) path = L"Software\\Microsoft\\Installer\\Win32Assemblies\\";
+        else path = L"Software\\Microsoft\\Installer\\Assemblies\\";
     }
     return RegCreateKeyW( root, path, hkey );
 }
@@ -578,12 +490,12 @@ static LONG open_local_assembly_key( UINT context, BOOL win32, const WCHAR *file
 
     if ((res = open_assemblies_key( context, win32, &root )))
     {
-        msi_free( path );
+        free( path );
         return res;
     }
     res = RegCreateKeyW( root, path, hkey );
     RegCloseKey( root );
-    msi_free( path );
+    free( path );
     return res;
 }
 
@@ -598,44 +510,31 @@ static LONG delete_local_assembly_key( UINT context, BOOL win32, const WCHAR *fi
 
     if ((res = open_assemblies_key( context, win32, &root )))
     {
-        msi_free( path );
+        free( path );
         return res;
     }
     res = RegDeleteKeyW( root, path );
     RegCloseKey( root );
-    msi_free( path );
+    free( path );
     return res;
 }
 
 static LONG open_global_assembly_key( UINT context, BOOL win32, HKEY *hkey )
 {
-    static const WCHAR path_win32[] =
-        {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
-         'I','n','s','t','a','l','l','e','r','\\','W','i','n','3','2','A','s','s','e','m','b','l','i','e','s','\\',
-         'G','l','o','b','a','l',0};
-    static const WCHAR path_dotnet[] =
-        {'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
-         'I','n','s','t','a','l','l','e','r','\\','A','s','s','e','m','b','l','i','e','s','\\',
-         'G','l','o','b','a','l',0};
-    static const WCHAR classes_path_win32[] =
-        {'I','n','s','t','a','l','l','e','r','\\','W','i','n','3','2','A','s','s','e','m','b','l','i','e','s','\\',
-         'G','l','o','b','a','l',0};
-    static const WCHAR classes_path_dotnet[] =
-        {'I','n','s','t','a','l','l','e','r','\\','A','s','s','e','m','b','l','i','e','s','\\','G','l','o','b','a','l',0};
     HKEY root;
     const WCHAR *path;
 
     if (context == MSIINSTALLCONTEXT_MACHINE)
     {
         root = HKEY_CLASSES_ROOT;
-        if (win32) path = classes_path_win32;
-        else path = classes_path_dotnet;
+        if (win32) path = L"Installer\\Win32Assemblies\\Global";
+        else path = L"Installer\\Assemblies\\Global";
     }
     else
     {
         root = HKEY_CURRENT_USER;
-        if (win32) path = path_win32;
-        else path = path_dotnet;
+        if (win32) path = L"Software\\Microsoft\\Installer\\Win32Assemblies\\Global";
+        else path = L"Software\\Microsoft\\Installer\\Assemblies\\Global";
     }
     return RegCreateKeyW( root, path, hkey );
 }
@@ -645,7 +544,7 @@ UINT ACTION_MsiPublishAssemblies( MSIPACKAGE *package )
     MSICOMPONENT *comp;
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szMsiPublishAssemblies);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"MsiPublishAssemblies");
 
     LIST_FOR_EACH_ENTRY(comp, &package->components, MSICOMPONENT, entry)
     {
@@ -686,7 +585,7 @@ UINT ACTION_MsiPublishAssemblies( MSIPACKAGE *package )
             }
             if ((res = open_local_assembly_key( package->Context, win32, file->TargetPath, &hkey )))
             {
-                WARN("failed to open local assembly key %d\n", res);
+                WARN( "failed to open local assembly key %ld\n", res );
                 return ERROR_FUNCTION_FAILED;
             }
         }
@@ -694,14 +593,14 @@ UINT ACTION_MsiPublishAssemblies( MSIPACKAGE *package )
         {
             if ((res = open_global_assembly_key( package->Context, win32, &hkey )))
             {
-                WARN("failed to open global assembly key %d\n", res);
+                WARN( "failed to open global assembly key %ld\n", res );
                 return ERROR_FUNCTION_FAILED;
             }
         }
         size = sizeof(buffer);
         if ((res = RegSetValueExW( hkey, assembly->display_name, 0, REG_MULTI_SZ, (const BYTE *)buffer, size )))
         {
-            WARN("failed to set assembly value %d\n", res);
+            WARN( "failed to set assembly value %ld\n", res );
         }
         RegCloseKey( hkey );
 
@@ -718,7 +617,7 @@ UINT ACTION_MsiUnpublishAssemblies( MSIPACKAGE *package )
     MSICOMPONENT *comp;
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szMsiUnpublishAssemblies);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"MsiUnpublishAssemblies");
 
     LIST_FOR_EACH_ENTRY(comp, &package->components, MSICOMPONENT, entry)
     {
@@ -747,17 +646,17 @@ UINT ACTION_MsiUnpublishAssemblies( MSIPACKAGE *package )
                 continue;
             }
             if ((res = delete_local_assembly_key( package->Context, win32, file->TargetPath )))
-                WARN("failed to delete local assembly key %d\n", res);
+                WARN( "failed to delete local assembly key %ld\n", res );
         }
         else
         {
             HKEY hkey;
             if ((res = open_global_assembly_key( package->Context, win32, &hkey )))
-                WARN("failed to delete global assembly key %d\n", res);
+                WARN( "failed to delete global assembly key %ld\n", res );
             else
             {
                 if ((res = RegDeleteValueW( hkey, assembly->display_name )))
-                    WARN("failed to delete global assembly value %d\n", res);
+                    WARN( "failed to delete global assembly value %ld\n", res );
                 RegCloseKey( hkey );
             }
         }
