@@ -29,47 +29,41 @@
 
 #define RSAENH_MAX_HASH_SIZE        104
 
-typedef union tagHASH_CONTEXT {
-    BCRYPT_HASH_HANDLE bcrypt_hash;
-} HASH_CONTEXT;
-
 typedef union tagKEY_CONTEXT {
-    rc2_key rc2;
-    des_key des;
-    des3_key des3;
-    aes_key aes;
-    prng_state rc4;
+    symmetric_key key;
+    prng_state prng;
     rsa_key rsa;
 } KEY_CONTEXT;
 
-BOOL init_hash_impl(ALG_ID aiAlgid, HASH_CONTEXT *pHashContext) DECLSPEC_HIDDEN;
-BOOL update_hash_impl(HASH_CONTEXT *pHashContext, const BYTE *pbData,
-                      DWORD dwDataLen) DECLSPEC_HIDDEN;
-BOOL finalize_hash_impl(HASH_CONTEXT *pHashContext, BYTE *pbHashValue) DECLSPEC_HIDDEN;
-BOOL duplicate_hash_impl(const HASH_CONTEXT *pSrcHashContext,
-                         HASH_CONTEXT *pDestHashContext) DECLSPEC_HIDDEN;
+extern prng_state prng;
+extern int wprng;
 
-BOOL new_key_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext, DWORD dwKeyLen) DECLSPEC_HIDDEN;
-BOOL free_key_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext) DECLSPEC_HIDDEN;
+BOOL init_hash_impl(ALG_ID aiAlgid, BCRYPT_HASH_HANDLE *hash_handle);
+BOOL update_hash_impl(BCRYPT_HASH_HANDLE hash_handle, const BYTE *pbData, DWORD dwDataLen);
+BOOL finalize_hash_impl(BCRYPT_HASH_HANDLE hash_handle, BYTE *hash_value, DWORD hash_size);
+BOOL duplicate_hash_impl(BCRYPT_HASH_HANDLE src_hash_handle, BCRYPT_HASH_HANDLE *dest_hash_handle);
+
+BOOL new_key_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext, DWORD dwKeyLen);
+BOOL free_key_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext);
 BOOL setup_key_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext, DWORD dwKeyLen,
-                    DWORD dwEffectiveKeyLen, DWORD dwSaltLen, BYTE *abKeyValue) DECLSPEC_HIDDEN;
+                    DWORD dwEffectiveKeyLen, DWORD dwSaltLen, BYTE *abKeyValue);
 BOOL duplicate_key_impl(ALG_ID aiAlgid, const KEY_CONTEXT *pSrcKeyContext,
-                        KEY_CONTEXT *pDestKeyContext) DECLSPEC_HIDDEN;
+                        KEY_CONTEXT *pDestKeyContext);
 
 /* dwKeySpec is optional for symmetric key algorithms */
 BOOL encrypt_block_impl(ALG_ID aiAlgid, DWORD dwKeySpec, KEY_CONTEXT *pKeyContext, const BYTE *pbIn,
-                        BYTE *pbOut, DWORD enc) DECLSPEC_HIDDEN;
-BOOL encrypt_stream_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext, BYTE *pbInOut, DWORD dwLen) DECLSPEC_HIDDEN;
+                        BYTE *pbOut);
+BOOL decrypt_block_impl(ALG_ID aiAlgid, DWORD dwKeySpec, KEY_CONTEXT *pKeyContext, const BYTE *pbIn,
+                        BYTE *pbOut);
+BOOL encrypt_stream_impl(ALG_ID aiAlgid, KEY_CONTEXT *pKeyContext, BYTE *pbInOut, DWORD dwLen);
 
 BOOL export_public_key_impl(BYTE *pbDest, const KEY_CONTEXT *pKeyContext, DWORD dwKeyLen,
-                            DWORD *pdwPubExp) DECLSPEC_HIDDEN;
+                            DWORD *pdwPubExp);
 BOOL import_public_key_impl(const BYTE *pbSrc, KEY_CONTEXT *pKeyContext, DWORD dwKeyLen,
-                            DWORD dwPubExp) DECLSPEC_HIDDEN;
+                            DWORD dwPubExp);
 BOOL export_private_key_impl(BYTE *pbDest, const KEY_CONTEXT *pKeyContext, DWORD dwKeyLen,
-                             DWORD *pdwPubExp) DECLSPEC_HIDDEN;
+                             DWORD *pdwPubExp);
 BOOL import_private_key_impl(const BYTE* pbSrc, KEY_CONTEXT *pKeyContext, DWORD dwKeyLen,
-                             DWORD dwDataLen, DWORD dwPubExp) DECLSPEC_HIDDEN;
-
-BOOL gen_rand_impl(BYTE *pbBuffer, DWORD dwLen) DECLSPEC_HIDDEN;
+                             DWORD dwDataLen, DWORD dwPubExp);
 
 #endif /* __WINE_IMPLGLUE_H */
