@@ -54,7 +54,7 @@ HRESULT create_view( enum view_type type, enum wbm_namespace ns, const WCHAR *pa
 
     case VIEW_TYPE_SELECT:
     {
-        struct table *table = find_table( ns, class );
+        struct table *table = create_table( ns, class );
         HRESULT hr;
 
         if (table && (hr = append_table( view, table )) != S_OK)
@@ -94,23 +94,30 @@ void destroy_view( struct view *view )
     free( view );
 }
 
-static BOOL eval_like( const WCHAR *lstr, const WCHAR *rstr )
+static BOOL eval_like( const WCHAR *text, const WCHAR *pattern )
 {
-    const WCHAR *p = lstr, *q = rstr;
+    if (wcsstr( pattern, L"[" )) FIXME( "character ranges (i.e. [abc], [^a-z]) are not supported\n" );
 
-    while (*p && *q)
+    while (*text)
     {
-        if (q[0] == '\\' && q[1] == '\\') q++;
-        if (*q == '%')
+        if (pattern[0] == '\\' && pattern[1] == '\\') pattern++;
+
+        if (*pattern == '%')
         {
-            while (*q == '%') q++;
-            if (!*q) return TRUE;
-            while (*p && *q && towupper( *p ) == towupper( *q )) { p++; q++; };
-            if (!*p && !*q) return TRUE;
+            while (*pattern == '%') pattern++;
+            if (!*pattern) return TRUE;
+            while (!eval_like( text, pattern ) && *text) text++;
+
+            return !!*text;
         }
-        if (*q != '%' && towupper( *p++ ) != towupper( *q++ )) return FALSE;
+
+        if (*pattern != '_' && towupper( *text ) != towupper( *pattern )) return FALSE;
+        text++; pattern++;
     }
-    return TRUE;
+
+    while (*pattern == '%') pattern++;
+
+    return *pattern == '\0';
 }
 
 static HRESULT eval_strcmp( UINT op, const WCHAR *lstr, const WCHAR *rstr, LONGLONG *val )
@@ -695,12 +702,9 @@ static HRESULT exec_select_view( struct view *view )
     if (!view->table_count) return S_OK;
 
     table = view->table[0];
-    if (table->fill)
-    {
-        clear_table( table );
-        status = table->fill( table, view->cond );
-    }
+    if (table->fill) status = table->fill( table, view->cond );
     if (status == FILL_STATUS_FAILED) return WBEM_E_FAILED;
+
     if (!table->num_rows) return S_OK;
 
     len = min( table->num_rows, 16 );
