@@ -248,6 +248,7 @@ DC *alloc_dc_ptr( DWORD magic )
  */
 static void free_dc_state( DC *dc )
 {
+    if (dc->opengl_drawable) opengl_drawable_release( dc->opengl_drawable );
     if (dc->hClipRgn) NtGdiDeleteObjectApp( dc->hClipRgn );
     if (dc->hMetaRgn) NtGdiDeleteObjectApp( dc->hMetaRgn );
     if (dc->hVisRgn) NtGdiDeleteObjectApp( dc->hVisRgn );
@@ -458,7 +459,6 @@ void DC_UpdateXforms( DC *dc )
  */
 static BOOL reset_dc_state( HDC hdc )
 {
-    struct opengl_drawable *drawable;
     DC *dc, *dcs, *next;
 
     if (!(dc = get_dc_ptr( hdc ))) return FALSE;
@@ -487,12 +487,8 @@ static BOOL reset_dc_state( HDC hdc )
     }
     dc->saved_dc = NULL;
     dc->attr->save_level = 0;
-
-    drawable = dc->opengl_drawable;
-    dc->opengl_drawable = NULL;
     release_dc_ptr( dc );
 
-    if (drawable) opengl_drawable_release( drawable );
     return TRUE;
 }
 
@@ -1095,6 +1091,19 @@ BOOL WINAPI NtGdiSetBrushOrg( HDC hdc, INT x, INT y, POINT *oldorg )
 }
 
 
+BOOL offset_viewport_org( HDC hdc, INT x, INT y, POINT *point )
+{
+    DC *dc;
+
+    if (!(dc = get_dc_ptr( hdc ))) return FALSE;
+    if (point) *point = dc->attr->vport_org;
+    dc->attr->vport_org.x += x;
+    dc->attr->vport_org.y += y;
+    release_dc_ptr( dc );
+    return NtGdiComputeXformCoefficients( hdc );
+}
+
+
 BOOL set_viewport_org( HDC hdc, INT x, INT y, POINT *point )
 {
     DC *dc;
