@@ -106,7 +106,7 @@ static IOHIDManagerRef hid_manager;
 static CFRunLoopRef run_loop;
 static struct list event_queue = LIST_INIT(event_queue);
 static struct list device_list = LIST_INIT(device_list);
-static struct iohid_bus_options options;
+static const struct bus_options *options;
 
 struct iohid_device
 {
@@ -275,13 +275,32 @@ static void handle_DeviceMatchingCallback(void *context, IOReturn result, void *
         .serialnumber = {'0','0','0','0',0},
     };
     struct iohid_device *impl;
+    USAGE_AND_PAGE usages;
     CFStringRef str;
 
+    usages.UsagePage = CFNumberToDWORD(IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDPrimaryUsagePageKey)));
+    usages.Usage = CFNumberToDWORD(IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDPrimaryUsageKey)));
+
     desc.vid = CFNumberToDWORD(IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDVendorIDKey)));
     desc.pid = CFNumberToDWORD(IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDProductIDKey)));
     desc.version = CFNumberToDWORD(IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDVersionNumberKey)));
     desc.uid = CFNumberToDWORD(IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDLocationIDKey)));
 
+    if ((str = IOHIDDeviceGetProperty(IOHIDDevice, CFSTR(kIOHIDTransportKey))))
+        desc.is_bluetooth = !CFStringCompare(str, CFSTR(kIOHIDTransportBluetoothValue), 0) ||
+                            !CFStringCompare(str, CFSTR(kIOHIDTransportBluetoothLowEnergyValue), 0);
+
+    if (usages.UsagePage != HID_USAGE_PAGE_GENERIC ||
+        !(usages.Usage == HID_USAGE_GENERIC_JOYSTICK || usages.Usage == HID_USAGE_GENERIC_GAMEPAD))
+    {
+        /* winebus isn't currently meant to handle anything but these, and
+         * opening keyboards, mice, or the Touch Bar on older MacBooks triggers
+         * a permissions dialog for input monitoring.
+         */
+        WARN("Ignoring HID device %p (vid %04x, pid %04x): not a joystick or gamepad\n", IOHIDDevice, desc.vid, desc.pid);
+        return;
+    }
+
     if (IOHIDDeviceOpen(IOHIDDevice, 0) != kIOReturnSuccess)
     {
         ERR("Failed to open HID device %p (vid %04x, pid %04x)\n", IOHIDDevice, desc.vid, desc.pid);
@@ -301,44 +320,6 @@ static void handle_DeviceMatchingCallback(void *context, IOReturn result, void *
     {
         if (is_xbox_gamepad(desc.vid, desc.pid))
             desc.is_gamepad = TRUE;
-        else
-        {
-            int axes=0, buttons=0;
-            CFArrayRef element_array = IOHIDDeviceCopyMatchingElements(
-                IOHIDDevice, NULL, kIOHIDOptionsTypeNone);
-
-            if (element_array) {
-                CFIndex index;
-                CFIndex count = CFArrayGetCount(element_array);
-                for (index = 0; index < count; index++)
-                {
-                    IOHIDElementRef element = (IOHIDElementRef)CFArrayGetValueAtIndex(element_array, index);
-                    if (element)
-                    {
-                        int type = IOHIDElementGetType(element);
-                        if (type == kIOHIDElementTypeInput_Button) buttons++;
-                        if (type == kIOHIDElementTypeInput_Axis) axes++;
-                        if (type == kIOHIDElementTypeInput_Misc)
-                        {
-                            uint32_t usage = IOHIDElementGetUsage(element);
-                            switch (usage)
-                            {
-                                case kHIDUsage_GD_X:
-                                case kHIDUsage_GD_Y:
-                                case kHIDUsage_GD_Z:
-                                case kHIDUsage_GD_Rx:
-                                case kHIDUsage_GD_Ry:
-                                case kHIDUsage_GD_Rz:
-                                case kHIDUsage_GD_Slider:
-                                    axes ++;
-                            }
-                        }
-                    }
-                }
-                CFRelease(element_array);
-            }
-            desc.is_gamepad = (axes == 6  && buttons >= 14);
-        }
     }
 
     TRACE("dev %p, desc %s.\n", IOHIDDevice, debugstr_device_desc(&desc));
@@ -371,7 +352,7 @@ NTSTATUS iohid_bus_init(void *args)
 {
     TRACE("args %p\n", args);
 
-    options = *(struct iohid_bus_options *)args;
+    options = args;
 
     if (!(hid_manager = IOHIDManagerCreate(kCFAllocatorDefault, 0L)))
     {
