@@ -22,7 +22,6 @@
 
 #include "macdrv_cocoa.h"
 
-
 #define ERR(...) do { if (macdrv_err_on) LogError(__func__, __VA_ARGS__); } while (false)
 
 
@@ -33,6 +32,7 @@ enum {
 
 @class WineEventQueue;
 @class WineWindow;
+@protocol WineClipCursorHandler;
 
 
 @interface WineApplicationController : NSObject <NSApplicationDelegate>
@@ -55,7 +55,6 @@ enum {
     CGEventSourceKeyboardType keyboardType;
     NSEvent* lastFlagsChanged;
     BOOL inputSourceIsInputMethod;
-    BOOL inputSourceIsInputMethodValid;
     uint32_t pressedKeyCodes[128 / 32];
 
     CGFloat primaryScreenHeight;
@@ -83,13 +82,9 @@ enum {
     BOOL        cursorHidden;
     BOOL        clientWantsCursorHidden;
 
-    BOOL clippingCursor;
-    CGRect cursorClipRect;
-    CFMachPortRef cursorClippingEventTap;
-    NSMutableArray* warpRecords;
-    CGPoint synthesizedLocation;
     NSTimeInterval lastSetCursorPositionTime;
-    NSTimeInterval lastEventTapEventTime;
+
+    id<WineClipCursorHandler> clipCursorHandler;
 
     NSImage* applicationIcon;
 
@@ -103,10 +98,12 @@ enum {
 @property (readonly, nonatomic) BOOL areDisplaysCaptured;
 
 @property (readonly) BOOL clippingCursor;
+@property (nonatomic) NSTimeInterval lastSetCursorPositionTime;
 
     + (WineApplicationController*) sharedController;
 
-    - (void) transformProcessToForeground;
+    - (void) transformProcessToForeground:(BOOL)activateIfTransformed;
+    - (void) tryToActivateIgnoringOtherApps:(BOOL)ignore;
 
     - (BOOL) registerEventQueue:(WineEventQueue*)queue;
     - (void) unregisterEventQueue:(WineEventQueue*)queue;
@@ -124,6 +121,7 @@ enum {
     - (void) windowWillOrderOut:(WineWindow*)window;
 
     - (void) flipRect:(NSRect*)rect;
+    - (NSPoint) flippedMouseLocation:(NSPoint)point;
 
     - (WineWindow*) frontWineWindow;
     - (void) adjustWindowLevels;
