@@ -23,6 +23,8 @@
 #import "cocoa_cursorclipping.h"
 #import "cocoa_window.h"
 
+#pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
+
 
 /* Neither Quartz nor Cocoa has an exact analog for Win32 cursor clipping.
  *
@@ -230,7 +232,7 @@ static void scale_rect_for_retina_mode(int mode, CGRect *cursorClipRect)
 
             for (i = 0; i < warpsFinished; i++)
             {
-                WarpRecord* warpRecord = [warpRecords objectAtIndex:0];
+                WarpRecord* warpRecord = warpRecords[0];
                 deltaX -= warpRecord.to.x - warpRecord.from.x;
                 deltaY -= warpRecord.to.y - warpRecord.from.y;
                 [warpRecords removeObjectAtIndex:0];
@@ -426,14 +428,9 @@ static void scale_rect_for_retina_mode(int mode, CGRect *cursorClipRect)
 
     + (BOOL) isAvailable
     {
-        if ([NSProcessInfo instancesRespondToSelector:@selector(isOperatingSystemAtLeastVersion:)])
-        {
-            NSOperatingSystemVersion requiredVersion = { 10, 13, 0 };
-            return [[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:requiredVersion] &&
-                   [NSWindow instancesRespondToSelector:@selector(setMouseConfinementRect:)];
-        }
-
-        return FALSE;
+        NSOperatingSystemVersion requiredVersion = { 10, 13, 0 };
+        return [[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:requiredVersion] &&
+               [NSWindow instancesRespondToSelector:@selector(setMouseConfinementRect:)];
     }
 
     /* Returns the region of the given rect that intersects with the given
