@@ -31,6 +31,7 @@
 #include "winbase.h"
 #include "winreg.h"
 #include "x11drv.h"
+#include "xcomposite.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(x11drv);
@@ -255,6 +256,187 @@ HRGN get_dc_monitor_region( HWND hwnd, HDC hdc )
     return 0;
 }
 
+static const struct client_surface_funcs x11drv_client_surface_funcs;
+
+struct x11drv_client_surface
+{
+    struct client_surface client;
+    Window window;
+    RECT rect;
+
+    HDC hdc_src;
+    HDC hdc_dst;
+};
+
+static struct x11drv_client_surface *impl_from_client_surface( struct client_surface *client )
+{
+    return CONTAINING_RECORD( client, struct x11drv_client_surface, client );
+}
+
+static void x11drv_client_surface_destroy( struct client_surface *client )
+{
+    struct x11drv_client_surface *surface = impl_from_client_surface( client );
+    HWND hwnd = client->hwnd;
+
+    TRACE( "%s\n", debugstr_client_surface( client ) );
+
+    if (surface->window) destroy_client_window( hwnd, surface->window );
+    if (surface->hdc_dst) NtGdiDeleteObjectApp( surface->hdc_dst );
+    if (surface->hdc_src) NtGdiDeleteObjectApp( surface->hdc_src );
+}
+
+static void x11drv_client_surface_detach( struct client_surface *client )
+{
+    struct x11drv_client_surface *surface = impl_from_client_surface( client );
+    Window client_window = surface->window;
+    struct x11drv_win_data *data;
+    HWND hwnd = client->hwnd;
+
+    TRACE( "%s\n", debugstr_client_surface( client ) );
+
+    if ((data = get_win_data( hwnd )))
+    {
+        detach_client_window( data, client_window );
+        release_win_data( data );
+    }
+}
+
+static void client_surface_update_size( HWND hwnd, struct x11drv_client_surface *surface )
+{
+    XWindowChanges changes;
+    RECT rect;
+
+    NtUserGetClientRect( hwnd, &rect, NtUserGetDpiForWindow( hwnd ) );
+    if (EqualRect( &surface->rect, &rect )) return;
+
+    changes.width  = min( max( 1, rect.right ), 65535 );
+    changes.height = min( max( 1, rect.bottom ), 65535 );
+    XConfigureWindow( gdi_display, surface->window, CWWidth | CWHeight, &changes );
+    surface->rect = rect;
+}
+
+static void client_surface_update_offscreen( HWND hwnd, struct x11drv_client_surface *surface )
+{
+    BOOL offscreen = needs_offscreen_rendering( hwnd );
+    struct x11drv_win_data *data;
+
+    if (InterlockedExchange( &surface->client.offscreen, offscreen ) == offscreen)
+    {
+        if (!offscreen && (data = get_win_data( hwnd )))
+        {
+            attach_client_window( data, surface->window );
+            release_win_data( data );
+        }
+        return;
+    }
+
+    if (!offscreen)
+    {
+#ifdef SONAME_LIBXCOMPOSITE
+        if (usexcomposite) pXCompositeUnredirectWindow( gdi_display, surface->window, CompositeRedirectManual );
+#endif
+        if (surface->hdc_dst)
+        {
+            NtGdiDeleteObjectApp( surface->hdc_dst );
+            surface->hdc_dst = NULL;
+        }
+        if (surface->hdc_src)
+        {
+            NtGdiDeleteObjectApp( surface->hdc_src );
+            surface->hdc_src = NULL;
+        }
+    }
+    else
+    {
+        static const WCHAR displayW[] = {'D','I','S','P','L','A','Y'};
+        UNICODE_STRING device_str = RTL_CONSTANT_STRING(displayW);
+        surface->hdc_dst = NtGdiOpenDCW( &device_str, NULL, NULL, 0, TRUE, NULL, NULL, NULL );
+        surface->hdc_src = NtGdiOpenDCW( &device_str, NULL, NULL, 0, TRUE, NULL, NULL, NULL );
+        set_dc_drawable( surface->hdc_src, surface->window, &surface->rect, IncludeInferiors );
+#ifdef SONAME_LIBXCOMPOSITE
+        if (usexcomposite) pXCompositeRedirectWindow( gdi_display, surface->window, CompositeRedirectManual );
+#endif
+    }
+
+    if ((data = get_win_data( hwnd )))
+    {
+        if (offscreen) detach_client_window( data, surface->window );
+        else attach_client_window( data, surface->window );
+        release_win_data( data );
+    }
+}
+
+static void x11drv_client_surface_update( struct client_surface *client )
+{
+    struct x11drv_client_surface *surface = impl_from_client_surface( client );
+    HWND hwnd = client->hwnd;
+
+    TRACE( "%s\n", debugstr_client_surface( client ) );
+
+    client_surface_update_size( hwnd, surface );
+    client_surface_update_offscreen( hwnd, surface );
+}
+
+static void X11DRV_client_surface_present( struct client_surface *client, HDC hdc )
+{
+    struct x11drv_client_surface *surface = impl_from_client_surface( client );
+    HWND hwnd = client->hwnd, toplevel = NtUserGetAncestor( hwnd, GA_ROOT );
+    struct x11drv_win_data *data;
+    RECT rect_dst, rect;
+    Drawable window;
+    HRGN region;
+
+    client_surface_update_size( hwnd, surface );
+    client_surface_update_offscreen( hwnd, surface );
+
+    if (!hdc) return;
+    window = X11DRV_get_whole_window( toplevel );
+    region = get_dc_monitor_region( hwnd, hdc );
+
+    NtUserGetClientRect( hwnd, &rect_dst, NtUserGetWinMonitorDpi( hwnd, MDT_RAW_DPI ) );
+    NtUserMapWindowPoints( hwnd, toplevel, (POINT *)&rect_dst, 2, NtUserGetWinMonitorDpi( hwnd, MDT_RAW_DPI ) );
+
+    if ((data = get_win_data( toplevel )))
+    {
+        OffsetRect( &rect_dst, data->rects.client.left - data->rects.visible.left,
+                    data->rects.client.top - data->rects.visible.top );
+        release_win_data( data );
+    }
+
+    if (get_dc_drawable( surface->hdc_dst, &rect ) != window || !EqualRect( &rect, &rect_dst ))
+        set_dc_drawable( surface->hdc_dst, window, &rect_dst, IncludeInferiors );
+    if (region) NtGdiExtSelectClipRgn( surface->hdc_dst, region, RGN_COPY );
+
+    NtGdiStretchBlt( surface->hdc_dst, 0, 0, rect_dst.right - rect_dst.left, rect_dst.bottom - rect_dst.top,
+                     surface->hdc_src, 0, 0, surface->rect.right, surface->rect.bottom, SRCCOPY, 0 );
+
+    if (region) NtGdiDeleteObjectApp( region );
+}
+
+static const struct client_surface_funcs x11drv_client_surface_funcs =
+{
+    .destroy = x11drv_client_surface_destroy,
+    .detach = x11drv_client_surface_detach,
+    .update = x11drv_client_surface_update,
+    .present = X11DRV_client_surface_present,
+};
+
+Window x11drv_client_surface_create( HWND hwnd, const XVisualInfo *visual, Colormap colormap, struct client_surface **client )
+{
+    struct x11drv_client_surface *surface;
+
+    if (!(surface = client_surface_create( sizeof(*surface), &x11drv_client_surface_funcs, hwnd ))) return None;
+    if (!(surface->window = create_client_window( hwnd, visual, colormap )))
+    {
+        client_surface_release( &surface->client );
+        return None;
+    }
+    NtUserGetClientRect( hwnd, &surface->rect, NtUserGetDpiForWindow( hwnd ) );
+
+    *client = &surface->client;
+    return surface->window;
+}
+
 /**********************************************************************
  *           ExtEscape  (X11DRV.@)
  */
