@@ -23,15 +23,13 @@
 #ifndef _WINE_INTERNET_H_
 #define _WINE_INTERNET_H_
 
-#include "wine/unicode.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 #include <time.h>
 
 #include "winineti.h"
 
-extern HMODULE WININET_hModule DECLSPEC_HIDDEN;
+extern HMODULE WININET_hModule;
 
 typedef struct {
     WCHAR *name;
@@ -54,15 +52,15 @@ typedef struct {
     struct list conn_pool;
 } server_t;
 
-void server_addref(server_t*) DECLSPEC_HIDDEN;
-void server_release(server_t*) DECLSPEC_HIDDEN;
+void server_addref(server_t*);
+void server_release(server_t*);
 
 typedef enum {
     COLLECT_TIMEOUT,
     COLLECT_CONNECTIONS,
     COLLECT_CLEANUP
 } collect_type_t;
-BOOL collect_connections(collect_type_t) DECLSPEC_HIDDEN;
+BOOL collect_connections(collect_type_t);
 
 /* used for netconnection.c stuff */
 typedef struct
@@ -87,46 +85,10 @@ typedef struct
     struct list pool_entry;
 } netconn_t;
 
-BOOL is_valid_netconn(netconn_t *) DECLSPEC_HIDDEN;
-void close_netconn(netconn_t *) DECLSPEC_HIDDEN;
+BOOL is_valid_netconn(netconn_t *);
+void close_netconn(netconn_t *);
 
-static inline void * __WINE_ALLOC_SIZE(2) heap_realloc_zero(void *mem, size_t len)
-{
-    return HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, mem, len);
-}
-
-static inline LPWSTR heap_strdupW(LPCWSTR str)
-{
-    LPWSTR ret = NULL;
-
-    if(str) {
-        DWORD size;
-
-        size = (strlenW(str)+1)*sizeof(WCHAR);
-        ret = heap_alloc(size);
-        if(ret)
-            memcpy(ret, str, size);
-    }
-
-    return ret;
-}
-
-static inline char *heap_strdupA(const char *str)
-{
-    char *ret = NULL;
-
-    if(str) {
-        DWORD size = strlen(str)+1;
-
-        ret = heap_alloc(size);
-        if(ret)
-            memcpy(ret, str, size);
-    }
-
-    return ret;
-}
-
-static inline LPWSTR heap_strndupW(LPCWSTR str, UINT max_len)
+static inline WCHAR *strndupW(const WCHAR *str, UINT max_len)
 {
     LPWSTR ret;
     UINT len;
@@ -138,7 +100,7 @@ static inline LPWSTR heap_strndupW(LPCWSTR str, UINT max_len)
         if(str[len] == '\0')
             break;
 
-    ret = heap_alloc(sizeof(WCHAR)*(len+1));
+    ret = malloc(sizeof(WCHAR) * (len + 1));
     if(ret) {
         memcpy(ret, str, sizeof(WCHAR)*len);
         ret[len] = '\0';
@@ -147,15 +109,18 @@ static inline LPWSTR heap_strndupW(LPCWSTR str, UINT max_len)
     return ret;
 }
 
-static inline WCHAR *heap_strndupAtoW(const char *str, int len_a, DWORD *len_w)
+static inline WCHAR *strndupAtoW(const char *str, int len_a, DWORD *len_w)
 {
     WCHAR *ret = NULL;
 
     if(str) {
         size_t len;
-        if(len_a < 0) len_a = strlen(str);
+        if(len_a < 0)
+            len_a = strlen(str);
+        else if(len_a > 0)
+            len_a = strnlen(str, len_a);
         len = MultiByteToWideChar(CP_ACP, 0, str, len_a, NULL, 0);
-        ret = heap_alloc((len+1)*sizeof(WCHAR));
+        ret = malloc((len + 1) * sizeof(WCHAR));
         if(ret) {
             MultiByteToWideChar(CP_ACP, 0, str, len_a, ret, len);
             ret[len] = 0;
@@ -166,7 +131,7 @@ static inline WCHAR *heap_strndupAtoW(const char *str, int len_a, DWORD *len_w)
     return ret;
 }
 
-static inline WCHAR *heap_strdupAtoW(const char *str)
+static inline WCHAR *strdupAtoW(const char *str)
 {
     LPWSTR ret = NULL;
 
@@ -174,7 +139,7 @@ static inline WCHAR *heap_strdupAtoW(const char *str)
         DWORD len;
 
         len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
-        ret = heap_alloc(len*sizeof(WCHAR));
+        ret = malloc(len * sizeof(WCHAR));
         if(ret)
             MultiByteToWideChar(CP_ACP, 0, str, -1, ret, len);
     }
@@ -182,13 +147,13 @@ static inline WCHAR *heap_strdupAtoW(const char *str)
     return ret;
 }
 
-static inline char *heap_strdupWtoA(LPCWSTR str)
+static inline char *strdupWtoA(const WCHAR *str)
 {
     char *ret = NULL;
 
     if(str) {
         DWORD size = WideCharToMultiByte(CP_ACP, 0, str, -1, NULL, 0, NULL, NULL);
-        ret = heap_alloc(size);
+        ret = malloc(size);
         if(ret)
             WideCharToMultiByte(CP_ACP, 0, str, -1, ret, size, NULL, NULL);
     }
@@ -209,7 +174,7 @@ static inline substr_t substr(const WCHAR *str, size_t len)
 
 static inline substr_t substrz(const WCHAR *str)
 {
-    return substr(str, strlenW(str));
+    return substr(str, lstrlenW(str));
 }
 
 static inline void WININET_find_data_WtoA(LPWIN32_FIND_DATAW dataW, LPWIN32_FIND_DATAA dataA)
@@ -260,6 +225,7 @@ typedef struct {
     void (*CloseConnection)(object_header_t*);
     DWORD (*QueryOption)(object_header_t*,DWORD,void*,DWORD*,BOOL);
     DWORD (*SetOption)(object_header_t*,DWORD,void*,DWORD);
+    DWORD (*SetFilePointer)(object_header_t*,LONG,DWORD);
     DWORD (*ReadFile)(object_header_t*,void*,DWORD,DWORD*,DWORD,DWORD_PTR);
     DWORD (*WriteFile)(object_header_t*,const void*,DWORD,DWORD*);
     DWORD (*QueryDataAvailable)(object_header_t*,DWORD*,DWORD,DWORD_PTR);
@@ -274,6 +240,11 @@ struct _object_header_t
     WH_TYPE htype;
     const object_vtbl_t *vtbl;
     HINTERNET hInternet;
+    ULONG  connect_timeout;
+    ULONG  send_timeout;
+    ULONG  receive_timeout;
+    ULONG  data_send_timeout;
+    ULONG  data_receive_timeout;
     BOOL valid_handle;
     DWORD  dwFlags;
     DWORD_PTR dwContext;
@@ -296,7 +267,6 @@ typedef struct
     LPWSTR  proxyUsername;
     LPWSTR  proxyPassword;
     DWORD   accessType;
-    DWORD   connect_timeout;
 } appinfo_t;
 
 typedef struct
@@ -307,9 +277,6 @@ typedef struct
     LPWSTR  userName;
     LPWSTR  password;
     INTERNET_PORT hostPort; /* the final destination port of the request */
-    DWORD connect_timeout;
-    DWORD send_timeout;
-    DWORD receive_timeout;
 } http_session_t;
 
 #define HDR_ISREQUEST		0x0001
@@ -335,8 +302,8 @@ typedef struct {
 
 typedef struct {
     data_stream_t data_stream;
-    DWORD content_length;
-    DWORD content_read;
+    ULONGLONG content_length;
+    ULONGLONG content_read;
 } netconn_stream_t;
 
 #define READ_BUFFER_SIZE 8192
@@ -351,14 +318,12 @@ typedef struct
     LPWSTR verb;
     netconn_t *netconn;
     DWORD security_flags;
-    DWORD connect_timeout;
-    DWORD send_timeout;
-    DWORD receive_timeout;
     LPWSTR version;
     DWORD status_code;
     LPWSTR statusText;
     DWORD bytesToWrite;
     DWORD bytesWritten;
+    BOOL clear_auth; /* Flag to clear the password field on the authorization dialog */
 
     CRITICAL_SECTION headers_section;  /* section to protect the headers array */
     HTTPHEADERW *custHeaders;
@@ -366,13 +331,15 @@ typedef struct
 
     FILETIME last_modified;
     HANDLE hCacheFile;
+    ULONGLONG cache_size;  /* size of cached data */
     req_file_t *req_file;
     FILETIME expires;
     struct HttpAuthInfo *authInfo;
     struct HttpAuthInfo *proxyAuthInfo;
 
     CRITICAL_SECTION read_section;  /* section to protect the following fields */
-    DWORD contentLength;  /* total number of bytes to be read */
+    ULONGLONG contentLength;  /* total number of bytes to be read */
+    ULONGLONG content_pos;    /* content read position */
     BOOL  read_gzip;      /* are we reading in gzip mode? */
     DWORD read_pos;       /* current read position in read_buf */
     DWORD read_size;      /* valid data size in read_buf */
@@ -391,61 +358,61 @@ struct task_header_t
     object_header_t *hdr;
 };
 
-void *alloc_async_task(object_header_t*,async_task_proc_t,size_t) DECLSPEC_HIDDEN;
+void *alloc_async_task(object_header_t*,async_task_proc_t,size_t);
 
-void *alloc_object(object_header_t*,const object_vtbl_t*,size_t) DECLSPEC_HIDDEN;
-object_header_t *get_handle_object( HINTERNET hinternet ) DECLSPEC_HIDDEN;
-object_header_t *WININET_AddRef( object_header_t *info ) DECLSPEC_HIDDEN;
-BOOL WININET_Release( object_header_t *info ) DECLSPEC_HIDDEN;
+void *alloc_object(object_header_t*,const object_vtbl_t*,size_t);
+object_header_t *get_handle_object( HINTERNET hinternet );
+object_header_t *WININET_AddRef( object_header_t *info );
+BOOL WININET_Release( object_header_t *info );
 
-DWORD INET_QueryOption(object_header_t*,DWORD,void*,DWORD*,BOOL) DECLSPEC_HIDDEN;
-DWORD INET_SetOption(object_header_t*,DWORD,void*,DWORD) DECLSPEC_HIDDEN;
+DWORD INET_QueryOption(object_header_t*,DWORD,void*,DWORD*,BOOL);
+DWORD INET_SetOption(object_header_t*,DWORD,void*,DWORD);
 
-time_t ConvertTimeString(LPCWSTR asctime) DECLSPEC_HIDDEN;
+time_t ConvertTimeString(LPCWSTR asctime);
 
 HINTERNET FTP_Connect(appinfo_t *hIC, LPCWSTR lpszServerName,
 	INTERNET_PORT nServerPort, LPCWSTR lpszUserName,
 	LPCWSTR lpszPassword, DWORD dwFlags, DWORD_PTR dwContext,
-	DWORD dwInternalFlags) DECLSPEC_HIDDEN;
+	DWORD dwInternalFlags);
 
 DWORD HTTP_Connect(appinfo_t*,LPCWSTR,
         INTERNET_PORT nServerPort, LPCWSTR lpszUserName,
         LPCWSTR lpszPassword, DWORD dwFlags, DWORD_PTR dwContext,
-        DWORD dwInternalFlags, HINTERNET*) DECLSPEC_HIDDEN;
+        DWORD dwInternalFlags, HINTERNET*);
 
-BOOL GetAddress(const WCHAR*,INTERNET_PORT,SOCKADDR*,int*,char*) DECLSPEC_HIDDEN;
+BOOL GetAddress(const WCHAR*,INTERNET_PORT,SOCKADDR*,int*,char*);
 
-DWORD get_cookie_header(const WCHAR*,const WCHAR*,WCHAR**) DECLSPEC_HIDDEN;
-DWORD set_cookie(substr_t,substr_t,substr_t,substr_t,DWORD) DECLSPEC_HIDDEN;
+DWORD get_cookie_header(const WCHAR*,const WCHAR*,WCHAR**);
+DWORD set_cookie(substr_t,substr_t,substr_t,substr_t,DWORD);
 
-void INTERNET_SetLastError(DWORD dwError) DECLSPEC_HIDDEN;
-DWORD INTERNET_GetLastError(void) DECLSPEC_HIDDEN;
-DWORD INTERNET_AsyncCall(task_header_t*) DECLSPEC_HIDDEN;
-LPSTR INTERNET_GetResponseBuffer(void) DECLSPEC_HIDDEN;
+void INTERNET_SetLastError(DWORD dwError);
+DWORD INTERNET_GetLastError(void);
+DWORD INTERNET_AsyncCall(task_header_t*);
+LPSTR INTERNET_GetResponseBuffer(void);
 
 VOID INTERNET_SendCallback(object_header_t *hdr, DWORD_PTR dwContext,
                            DWORD dwInternetStatus, LPVOID lpvStatusInfo,
-                           DWORD dwStatusInfoLength) DECLSPEC_HIDDEN;
-WCHAR *INTERNET_FindProxyForProtocol(LPCWSTR szProxy, LPCWSTR proto) DECLSPEC_HIDDEN;
+                           DWORD dwStatusInfoLength);
+WCHAR *INTERNET_FindProxyForProtocol(LPCWSTR szProxy, LPCWSTR proto);
 
-DWORD create_netconn(server_t*,DWORD,BOOL,DWORD,netconn_t**) DECLSPEC_HIDDEN;
-void free_netconn(netconn_t*) DECLSPEC_HIDDEN;
-void NETCON_unload(void) DECLSPEC_HIDDEN;
-DWORD NETCON_secure_connect(netconn_t*,server_t*) DECLSPEC_HIDDEN;
+DWORD create_netconn(server_t*,DWORD,BOOL,DWORD,netconn_t**);
+void free_netconn(netconn_t*);
+void NETCON_unload(void);
+DWORD NETCON_secure_connect(netconn_t*,server_t*);
 DWORD NETCON_send(netconn_t *connection, const void *msg, size_t len, int flags,
-		int *sent /* out */) DECLSPEC_HIDDEN;
-DWORD NETCON_recv(netconn_t*,void*,size_t,BOOL,int*) DECLSPEC_HIDDEN;
-BOOL NETCON_is_alive(netconn_t*) DECLSPEC_HIDDEN;
-LPCVOID NETCON_GetCert(netconn_t *connection) DECLSPEC_HIDDEN;
-int NETCON_GetCipherStrength(netconn_t*) DECLSPEC_HIDDEN;
-DWORD NETCON_set_timeout(netconn_t *connection, BOOL send, DWORD value) DECLSPEC_HIDDEN;
-int sock_send(int fd, const void *msg, size_t len, int flags) DECLSPEC_HIDDEN;
-int sock_recv(int fd, void *msg, size_t len, int flags) DECLSPEC_HIDDEN;
+		int *sent /* out */);
+DWORD NETCON_recv(netconn_t*,void*,size_t,BOOL,int*);
+BOOL NETCON_is_alive(netconn_t*);
+LPCVOID NETCON_GetCert(netconn_t *connection);
+int NETCON_GetCipherStrength(netconn_t*);
+DWORD NETCON_set_timeout(netconn_t *connection, BOOL send, DWORD value);
+int sock_send(int fd, const void *msg, size_t len, int flags);
+int sock_recv(int fd, void *msg, size_t len, int flags);
 
-server_t *get_server(substr_t,INTERNET_PORT,BOOL,BOOL) DECLSPEC_HIDDEN;
+server_t *get_server(substr_t,INTERNET_PORT,BOOL,BOOL);
 
-DWORD create_req_file(const WCHAR*,req_file_t**) DECLSPEC_HIDDEN;
-void req_file_release(req_file_t*) DECLSPEC_HIDDEN;
+DWORD create_req_file(const WCHAR*,req_file_t**);
+void req_file_release(req_file_t*);
 
 static inline req_file_t *req_file_addref(req_file_t *req_file)
 {
@@ -453,13 +420,14 @@ static inline req_file_t *req_file_addref(req_file_t *req_file)
     return req_file;
 }
 
-BOOL init_urlcache(void) DECLSPEC_HIDDEN;
-void free_urlcache(void) DECLSPEC_HIDDEN;
-void free_cookie(void) DECLSPEC_HIDDEN;
+BOOL init_urlcache(void);
+void free_urlcache(void);
+void free_cookie(void);
+void free_authorization_cache(void);
 
-void init_winsock(void) DECLSPEC_HIDDEN;
+void init_winsock(void);
 
-#define MAX_REPLY_LEN	 	0x5B4
+#define MAX_REPLY_LEN   0x1000
 
 /* Used for debugging - maybe need to be shared in the Wine debugging code ? */
 typedef struct
