@@ -245,6 +245,41 @@ static inline void push_dc_driver( PHYSDEV *dev, PHYSDEV physdev, const struct g
     *dev = physdev;
 }
 
+/* support for client surfaces */
+
+struct client_surface;
+struct client_surface_funcs
+{
+    void (*destroy)( struct client_surface *surface );
+    /* detach the surface from its window, called from window owner thread */
+    void (*detach)( struct client_surface *surface );
+    /* update the surface to match its window state, called from window owner thread */
+    void (*update)( struct client_surface *surface );
+    /* present the client surface if necessary, hdc != NULL when offscreen, called from render thread */
+    void (*present)( struct client_surface *surface, HDC hdc );
+};
+
+struct client_surface
+{
+    const struct client_surface_funcs *funcs;
+    struct list                        entry;          /* entry in win32u managed list */
+    LONG                               ref;            /* reference count */
+    HWND                               hwnd;           /* window the surface was created for */
+    LONG                               updated;        /* has been moved / resized / reparented */
+    LONG                               offscreen;      /* client window is offscreen */
+};
+
+W32KAPI void *client_surface_create( UINT size, const struct client_surface_funcs *funcs, HWND hwnd );
+W32KAPI void client_surface_add_ref( struct client_surface *surface );
+W32KAPI void client_surface_release( struct client_surface *surface );
+W32KAPI void client_surface_present( struct client_surface *surface, HDC hdc );
+
+static inline const char *debugstr_client_surface( struct client_surface *surface )
+{
+    if (!surface) return "(null)";
+    return wine_dbg_sprintf( "%p/%p", surface->hwnd, surface );
+}
+
 /* support for window surfaces */
 
 struct window_surface;
