@@ -648,7 +648,8 @@ static void* extract_rcdata (LPCSTR name, LPCSTR type, DWORD* size)
     HRSRC rsrc;
     HGLOBAL hdl;
     LPVOID addr;
-    
+
+    *size = 0;
     if (!(rsrc = FindResourceA(NULL, name, type)) ||
         !(*size = SizeofResource (0, rsrc)) ||
         !(hdl = LoadResource (0, rsrc)) ||
@@ -1163,6 +1164,23 @@ static void get_dll_path(HMODULE dll, char **path, char *filename)
     *path = xstrdup( dllpath );
 }
 
+static const char *get_compiler_version(void)
+{
+#ifdef __clang__
+# ifdef _MSC_VER
+    return strmake( "clang %s (msvc %u)", __clang_version__, _MSC_VER );
+# else
+    return strmake( "clang %s", __clang_version__ );
+# endif
+#elif defined __GNUC__
+    return strmake( "gcc %u.%u.%u (%s)", __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__, __VERSION__ );
+#elif defined _MSC_VER
+    return strmake( "msvc %u", _MSC_VER );
+#else
+    return "unknown";
+#endif
+}
+
 static BOOL CALLBACK
 extract_test_proc (HMODULE hModule, LPCSTR lpszType, LPSTR lpszName, LONG_PTR lParam)
 {
@@ -1285,8 +1303,6 @@ static char *
 run_tests (char *logname, char *outdir)
 {
     int i;
-    char *strres, *eol, *nextline;
-    DWORD strsize;
     char tmppath[MAX_PATH], tempdir[MAX_PATH+4];
     BOOL newdir;
     DWORD needed;
@@ -1348,23 +1364,9 @@ run_tests (char *logname, char *outdir)
     }
     xprintf ("Version 4\n");
     xprintf ("Tests from build %s\n", build_id[0] ? build_id : "-" );
-    xprintf ("Archive: -\n");  /* no longer used */
     xprintf ("Tag: %s\n", tag);
     xprintf ("Build info:\n");
-    strres = extract_rcdata ("BUILD_INFO", "STRINGRES", &strsize);
-    while (strres) {
-        eol = memchr (strres, '\n', strsize);
-        if (!eol) {
-            nextline = NULL;
-            eol = strres + strsize;
-        } else {
-            strsize -= eol - strres + 1;
-            nextline = strsize?eol+1:NULL;
-            if (eol > strres && *(eol-1) == '\r') eol--;
-        }
-        xprintf ("    %.*s\n", (int)(eol-strres), strres);
-        strres = nextline;
-    }
+    xprintf ("    Compiler: %s\n", get_compiler_version());
     xprintf ("Operating system version:\n");
     print_version ();
     print_language ();
