@@ -748,6 +748,7 @@ type_t *type_interface_define(type_t *iface, attr_list_t *attrs, type_t *inherit
     iface->details.iface->inherit = inherit;
     iface->details.iface->disp_inherit = NULL;
     iface->details.iface->async_iface = NULL;
+    iface->details.iface->runtime_class = NULL;
     iface->details.iface->requires = requires;
     define_type(iface, where);
     compute_method_indexes(iface);
@@ -845,6 +846,24 @@ type_t *type_runtimeclass_declare(char *name, struct namespace *namespace)
     return type;
 }
 
+static void set_constructor_runtimeclass(type_t *runtimeclass, attr_list_t *attrs)
+{
+    const attr_t *attr;
+    if (!attrs) return;
+    LIST_FOR_EACH_ENTRY(attr, attrs, const attr_t, entry)
+    {
+        if (attr->type == ATTR_ACTIVATABLE || attr->type == ATTR_COMPOSABLE)
+        {
+            const expr_t *value = attr->u.pval;
+            if (value->type == EXPR_MEMBER && value->u.var->declspec.type->type_type == TYPE_INTERFACE &&
+                value->u.var->declspec.type->details.iface)
+            {
+                value->u.var->declspec.type->details.iface->runtime_class = runtimeclass;
+            }
+        }
+    }
+}
+
 type_t *type_runtimeclass_define(type_t *runtimeclass, attr_list_t *attrs,
         typeref_list_t *ifaces, const struct location *where)
 {
@@ -853,10 +872,10 @@ type_t *type_runtimeclass_define(type_t *runtimeclass, attr_list_t *attrs,
 
     runtimeclass->attrs = check_runtimeclass_attrs(runtimeclass->name, attrs);
     runtimeclass->details.runtimeclass.ifaces = ifaces;
+    set_constructor_runtimeclass(runtimeclass, attrs);
     define_type(runtimeclass, where);
-    if (!type_runtimeclass_get_default_iface(runtimeclass, FALSE) &&
-        !get_attrp(runtimeclass->attrs, ATTR_STATIC))
-        error_loc("runtimeclass %s must have a default interface or static factory\n", runtimeclass->name);
+    if (!type_runtimeclass_get_ifaces(runtimeclass) && !get_attrp(runtimeclass->attrs, ATTR_STATIC))
+        error_loc("runtimeclass %s must have at least one interface or static factory\n", runtimeclass->name);
 
     if (ifaces) LIST_FOR_EACH_ENTRY(ref, ifaces, typeref_t, entry)
     {
@@ -894,6 +913,7 @@ type_t *type_apicontract_declare(char *name, struct namespace *namespace)
 type_t *type_apicontract_define(type_t *apicontract, attr_list_t *attrs, const struct location *where)
 {
     apicontract->attrs = check_apicontract_attrs(apicontract->name, attrs);
+    apicontract->attrs = append_attr(apicontract->attrs, attr_int(*where, ATTR_APICONTRACT, 1));
     define_type(apicontract, where);
     return apicontract;
 }
@@ -1173,9 +1193,9 @@ static type_t *replace_type_parameters_in_type(type_t *type, typeref_list_t *ori
         return type;
     case TYPE_ARRAY:
         t = replace_type_parameters_in_type(type->details.array.elem.type, orig, repl);
-        if (t == t->details.array.elem.type) return type;
+        if (t == type->details.array.elem.type) return type;
         type = duptype(type, 0);
-        t->details.array.elem.type = t;
+        type->details.array.elem.type = t;
         return type;
     case TYPE_FUNCTION:
         t = duptype(type, 0);
