@@ -55,6 +55,7 @@ typedef struct _user_type_t context_handle_t;
 typedef struct _user_type_t generic_handle_t;
 typedef struct _statement_t statement_t;
 typedef struct _warning_t warning_t;
+typedef struct _version_t version_t;
 
 typedef struct list attr_list_t;
 typedef struct list str_list_t;
@@ -74,6 +75,7 @@ enum attr_type
     ATTR_AGGREGATABLE,
     ATTR_ALLOCATE,
     ATTR_ANNOTATION,
+    ATTR_APICONTRACT, /* implicit attribute */
     ATTR_APPOBJECT,
     ATTR_ASYNC,
     ATTR_ASYNCUUID,
@@ -343,6 +345,8 @@ struct _attr_t {
     unsigned int ival;
     void *pval;
   } u;
+  /* metadata */
+  unsigned int md_member;
   /* parser-internal */
   struct list entry;
   struct location where;
@@ -364,7 +368,7 @@ struct _expr_t {
     double dval;
     const char *sval;
     const expr_t *ext;
-    decl_spec_t tref;
+    var_t *var;
   } u;
   const expr_t *ext2;
   int is_const;
@@ -402,6 +406,7 @@ struct iface_details
   struct _type_t *inherit;
   struct _type_t *disp_inherit;
   struct _type_t *async_iface;
+  struct _type_t *runtime_class;
   typeref_list_t *requires;
 };
 
@@ -496,31 +501,11 @@ enum type_type
     TYPE_DELEGATE,
 };
 
-enum
-{
-    MD_ATTR_CONTRACT,
-    MD_ATTR_FLAGS,
-    MD_ATTR_APICONTRACT,
-    MD_ATTR_CONTRACTVERSION,
-    MD_ATTR_VERSION,
-    MD_ATTR_UUID,
-    MD_ATTR_EXCLUSIVETO,
-    MD_ATTR_STATIC,
-    MD_ATTR_ACTIVATABLE,
-    MD_ATTR_THREADING,
-    MD_ATTR_MARSHALINGBEHAVIOR,
-    MD_ATTR_OVERLOAD,
-    MD_ATTR_DEFAULT_OVERLOAD,
-    MD_ATTR_DEPRECATED,
-    MD_ATTR_MAX,
-};
-
 struct metadata
 {
     unsigned int ref;
     unsigned int def;
     unsigned int extends;
-    unsigned int member[MD_ATTR_MAX];
     /* get/put methods */
     unsigned int class_property;
     unsigned int iface_property;
@@ -673,6 +658,11 @@ struct _statement_t {
     unsigned int is_defined : 1;
 };
 
+struct _version_t {
+    unsigned short major;
+    unsigned short minor;
+};
+
 struct _warning_t {
     int num;
     struct list entry;
