@@ -23,22 +23,9 @@
 #ifndef __WINE_BUILD_H
 #define __WINE_BUILD_H
 
-#ifndef __WINE_CONFIG_H
-# error You must include config.h to use this header
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-
-#ifndef max
-#define max(a,b)   (((a) > (b)) ? (a) : (b))
-#endif
-#ifndef min
-#define min(a,b)   (((a) < (b)) ? (a) : (b))
-#endif
-
-#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+#include "../tools.h"
 
 typedef enum
 {
@@ -114,6 +101,45 @@ typedef struct
     } u;
 } ORDDEF;
 
+struct apiset_entry
+{
+    unsigned int name_off;
+    unsigned int name_len;
+    unsigned int hash;
+    unsigned int hash_len;
+    unsigned int val_count;
+    struct apiset_value
+    {
+        unsigned int name_off;
+        unsigned int name_len;
+        unsigned int val_off;
+        unsigned int val_len;
+    } values[4];
+};
+
+struct apiset
+{
+    unsigned int count;
+    unsigned int size;
+    struct apiset_entry *entries;
+    unsigned int str_pos;
+    unsigned int str_size;
+    char *strings;
+};
+
+static const unsigned int apiset_hash_factor = 31;
+
+struct exports
+{
+    int              nb_entry_points;    /* number of used entry points */
+    ORDDEF         **entry_points;       /* dll entry points */
+    int              nb_names;           /* number of entry points with names */
+    ORDDEF         **names;              /* array of entry point names (points into entry_points) */
+    int              base;               /* ordinal base */
+    int              limit;              /* ordinal limit */
+    ORDDEF         **ordinals;           /* array of dll ordinals (points into entry_points) */
+};
+
 typedef struct
 {
     char            *src_name;           /* file name of the source spec file */
@@ -123,49 +149,36 @@ typedef struct
     char            *init_func;          /* initialization routine */
     char            *main_module;        /* main Win32 module for Win16 specs */
     SPEC_TYPE        type;               /* type of dll (Win16/Win32) */
-    int              base;               /* ordinal base */
-    int              limit;              /* ordinal limit */
     int              stack_size;         /* exe stack size */
     int              heap_size;          /* exe heap size */
     int              nb_entry_points;    /* number of used entry points */
     int              alloc_entry_points; /* number of allocated entry points */
-    int              nb_names;           /* number of entry points with names */
     unsigned int     nb_resources;       /* number of resources */
     int              characteristics;    /* characteristics for the PE header */
     int              dll_characteristics;/* DLL characteristics for the PE header */
     int              subsystem;          /* subsystem id */
     int              subsystem_major;    /* subsystem version major number */
     int              subsystem_minor;    /* subsystem version minor number */
-    ORDDEF          *entry_points;       /* dll entry points */
-    ORDDEF         **names;              /* array of entry point names (points into entry_points) */
-    ORDDEF         **ordinals;           /* array of dll ordinals (points into entry_points) */
+    int              unicode_app;        /* default to unicode entry point */
+    ORDDEF          *entry_points;       /* spec entry points */
+    struct exports   exports;            /* dll exports */
+    struct exports   native_exports;     /* dll native exports */
     struct resource *resources;          /* array of dll resources (format differs between Win16/Win32) */
+    struct apiset    apiset;             /* list of defined api sets */
 } DLLSPEC;
 
-enum target_cpu
-{
-    CPU_x86, CPU_x86_64, CPU_POWERPC, CPU_ARM, CPU_ARM64, CPU_LAST = CPU_ARM64
-};
-
-enum target_platform
-{
-    PLATFORM_UNSPECIFIED,
-    PLATFORM_APPLE,
-    PLATFORM_FREEBSD,
-    PLATFORM_SOLARIS,
-    PLATFORM_WINDOWS
-};
-
 extern char *target_alias;
-extern enum target_cpu target_cpu;
-extern enum target_platform target_platform;
+extern struct target target;
 
-struct strarray
+static inline unsigned int get_ptr_size(void)
 {
-    const char **str;
-    unsigned int count;
-    unsigned int max;
-};
+    return get_target_ptr_size( target );
+}
+
+static inline int is_pe(void)
+{
+    return is_pe_target( target );
+}
 
 /* entry point flags */
 #define FLAG_NORELAY   0x0001  /* don't use relay debugging for this function */
@@ -175,18 +188,19 @@ struct strarray
 #define FLAG_REGISTER  0x0010  /* use register calling convention */
 #define FLAG_PRIVATE   0x0020  /* function is private (cannot be imported) */
 #define FLAG_ORDINAL   0x0040  /* function should be imported by ordinal */
-#define FLAG_THISCALL  0x0080  /* use thiscall calling convention */
-#define FLAG_FASTCALL  0x0100  /* use fastcall calling convention */
-#define FLAG_IMPORT    0x0200  /* export is imported from another module */
+#define FLAG_THISCALL  0x0080  /* function uses thiscall calling convention */
+#define FLAG_FASTCALL  0x0100  /* function uses fastcall calling convention */
+#define FLAG_SYSCALL   0x0200  /* function is a system call */
+#define FLAG_IMPORT    0x0400  /* export is imported from another module */
 
 #define FLAG_FORWARD   0x1000  /* function is a forwarded name */
 #define FLAG_EXT_LINK  0x2000  /* function links to an external symbol */
 #define FLAG_EXPORT32  0x4000  /* 32-bit export in 16-bit spec file */
 
 #define FLAG_CPU(cpu)  (0x10000 << (cpu))
-#define FLAG_CPU_MASK  (FLAG_CPU(CPU_LAST + 1) - FLAG_CPU(0))
-#define FLAG_CPU_WIN64 (FLAG_CPU(CPU_x86_64) | FLAG_CPU(CPU_ARM64))
-#define FLAG_CPU_WIN32 (FLAG_CPU_MASK & ~FLAG_CPU_WIN64)
+#define FLAG_CPU_MASK  (FLAG_CPU_WIN32 | FLAG_CPU_WIN64)
+#define FLAG_CPU_WIN64 (FLAG_CPU(CPU_x86_64) | FLAG_CPU(CPU_ARM64) | FLAG_CPU(CPU_ARM64EC))
+#define FLAG_CPU_WIN32 (FLAG_CPU(CPU_i386) | FLAG_CPU(CPU_ARM))
 
 #define MAX_ORDINALS  65535
 
@@ -209,7 +223,18 @@ struct strarray
 #define IMAGE_FILE_UP_SYSTEM_ONLY	   0x4000
 #define IMAGE_FILE_BYTES_REVERSED_HI	   0x8000
 
-#define IMAGE_DLLCHARACTERISTICS_NX_COMPAT 0x0100
+#define IMAGE_DLLCHARACTERISTICS_PREFER_NATIVE         0x0010 /* Wine extension */
+#define IMAGE_DLLCHARACTERISTICS_HIGH_ENTROPY_VA       0x0020
+#define IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE          0x0040
+#define IMAGE_DLLCHARACTERISTICS_FORCE_INTEGRITY       0x0080
+#define IMAGE_DLLCHARACTERISTICS_NX_COMPAT             0x0100
+#define IMAGE_DLLCHARACTERISTICS_NO_ISOLATION          0x0200
+#define IMAGE_DLLCHARACTERISTICS_NO_SEH                0x0400
+#define IMAGE_DLLCHARACTERISTICS_NO_BIND               0x0800
+#define IMAGE_DLLCHARACTERISTICS_APPCONTAINER          0x1000
+#define IMAGE_DLLCHARACTERISTICS_WDM_DRIVER            0x2000
+#define IMAGE_DLLCHARACTERISTICS_GUARD_CF              0x4000
+#define IMAGE_DLLCHARACTERISTICS_TERMINAL_SERVER_AWARE 0x8000
 
 #define	IMAGE_SUBSYSTEM_NATIVE      1
 #define	IMAGE_SUBSYSTEM_WINDOWS_GUI 2
@@ -218,10 +243,6 @@ struct strarray
 
 /* global functions */
 
-#ifndef __GNUC__
-#define __attribute__(X)
-#endif
-
 #ifndef DECLSPEC_NORETURN
 # if defined(_MSC_VER) && (_MSC_VER >= 1200) && !defined(MIDL_PASS)
 #  define DECLSPEC_NORETURN __declspec(noreturn)
@@ -229,17 +250,7 @@ struct strarray
 #  define DECLSPEC_NORETURN __attribute__((noreturn))
 # endif
 #endif
-
-extern void *xmalloc (size_t size);
-extern void *xrealloc (void *ptr, size_t size);
-extern char *xstrdup( const char *str );
 extern char *strupper(char *s);
-extern int strendswith(const char* str, const char* end);
-extern char *strmake(const char* fmt, ...) __attribute__((__format__ (__printf__, 1, 2 )));
-extern struct strarray strarray_fromstring( const char *str, const char *delim );
-extern void strarray_add( struct strarray *array, ... );
-extern void strarray_addv( struct strarray *array, char * const *argv );
-extern void strarray_addall( struct strarray *array, struct strarray args );
 extern DECLSPEC_NORETURN void fatal_error( const char *msg, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
 extern DECLSPEC_NORETURN void fatal_perror( const char *msg, ... )
@@ -252,15 +263,18 @@ extern int output( const char *format, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
 extern void output_cfi( const char *format, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
+extern void output_seh( const char *format, ... )
+   __attribute__ ((__format__ (__printf__, 1, 2)));
 extern void output_rva( const char *format, ... )
    __attribute__ ((__format__ (__printf__, 1, 2)));
+extern void output_thunk_rva( int ordinal, const char *format, ... )
+   __attribute__ ((__format__ (__printf__, 2, 3)));
 extern void spawn( struct strarray array );
 extern struct strarray find_tool( const char *name, const char * const *names );
+extern struct strarray find_link_tool(void);
 extern struct strarray get_as_command(void);
 extern struct strarray get_ld_command(void);
 extern const char *get_nm_command(void);
-extern void cleanup_tmp_files(void);
-extern char *get_temp_file_name( const char *prefix, const char *suffix );
 extern void output_standard_file_header(void);
 extern FILE *open_input_file( const char *srcdir, const char *name );
 extern void close_input_file( FILE *file );
@@ -273,15 +287,13 @@ extern void assemble_file( const char *src_file, const char *obj_file );
 extern DLLSPEC *alloc_dll_spec(void);
 extern void free_dll_spec( DLLSPEC *spec );
 extern char *make_c_identifier( const char *str );
-extern const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec );
+extern const char *get_abi_name( const ORDDEF *odp, const char *name );
 extern const char *get_link_name( const ORDDEF *odp );
-extern int get_cpu_from_name( const char *name );
-extern unsigned int get_alignment(unsigned int align);
-extern unsigned int get_page_size(void);
-extern unsigned int get_ptr_size(void);
+extern int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const void *) );
+extern unsigned int get_section_alignment(void);
 extern unsigned int get_args_size( const ORDDEF *odp );
 extern const char *asm_name( const char *func );
-extern const char *func_declaration( const char *func );
+extern const char *arm64_name( const char *func );
 extern const char *asm_globl( const char *func );
 extern const char *get_asm_ptr_keyword(void);
 extern const char *get_asm_string_keyword(void);
@@ -289,28 +301,33 @@ extern const char *get_asm_export_section(void);
 extern const char *get_asm_rodata_section(void);
 extern const char *get_asm_rsrc_section(void);
 extern const char *get_asm_string_section(void);
+extern void output_function_header( const char *func, int global );
 extern void output_function_size( const char *name );
 extern void output_gnu_stack_note(void);
 
-extern void add_import_dll( const char *name, const char *filename );
 extern void add_delayed_import( const char *name );
 extern void add_extra_ld_symbol( const char *name );
-extern void read_undef_symbols( DLLSPEC *spec, char **argv );
+extern void add_spec_extra_ld_symbol( const char *name );
+extern void read_undef_symbols( DLLSPEC *spec, struct strarray files );
 extern void resolve_imports( DLLSPEC *spec );
 extern int is_undefined( const char *name );
 extern int has_imports(void);
+extern int has_delay_imports(void);
 extern void output_get_pc_thunk(void);
 extern void output_module( DLLSPEC *spec );
 extern void output_stubs( DLLSPEC *spec );
 extern void output_imports( DLLSPEC *spec );
-extern void output_import_lib( DLLSPEC *spec, char **argv );
+extern void output_import_lib( DLLSPEC *spec, struct strarray files );
+extern void output_static_lib( const char *output_name, struct strarray files, int create );
 extern void output_exports( DLLSPEC *spec );
 extern int load_res32_file( const char *name, DLLSPEC *spec );
 extern void output_resources( DLLSPEC *spec );
 extern void output_bin_resources( DLLSPEC *spec, unsigned int start_rva );
 extern void output_spec32_file( DLLSPEC *spec );
 extern void output_fake_module( DLLSPEC *spec );
-extern void output_def_file( DLLSPEC *spec, int import_only );
+extern void output_data_module( DLLSPEC *spec );
+extern void output_def_file( DLLSPEC *spec, struct exports *exports, int import_only );
+extern void output_apiset_lib( DLLSPEC *spec, const struct apiset *apiset );
 extern void load_res16_file( const char *name, DLLSPEC *spec );
 extern void output_res16_data( DLLSPEC *spec );
 extern void output_bin_res16_data( DLLSPEC *spec );
@@ -320,7 +337,8 @@ extern void output_spec16_file( DLLSPEC *spec );
 extern void output_fake_module16( DLLSPEC *spec16 );
 extern void output_res_o_file( DLLSPEC *spec );
 extern void output_asm_relays16(void);
-extern void make_builtin_files( char *argv[] );
+extern void make_builtin_files( struct strarray files );
+extern void fixup_constructors( struct strarray files );
 
 extern void add_16bit_exports( DLLSPEC *spec32, DLLSPEC *spec16 );
 extern int parse_spec_file( FILE *file, DLLSPEC *spec );
@@ -333,23 +351,12 @@ extern const char *input_buffer_filename;
 extern const unsigned char *input_buffer;
 extern size_t input_buffer_pos;
 extern size_t input_buffer_size;
-extern unsigned char *output_buffer;
-extern size_t output_buffer_pos;
-extern size_t output_buffer_size;
 
 extern void init_input_buffer( const char *file );
-extern void init_output_buffer(void);
-extern void flush_output_buffer(void);
 extern unsigned char get_byte(void);
 extern unsigned short get_word(void);
 extern unsigned int get_dword(void);
-extern void put_data( const void *data, size_t size );
-extern void put_byte( unsigned char val );
-extern void put_word( unsigned short val );
-extern void put_dword( unsigned int val );
-extern void put_qword( unsigned int val );
 extern void put_pword( unsigned int val );
-extern void align_output( unsigned int align );
 
 /* global variables */
 
@@ -362,14 +369,18 @@ extern int verbose;
 extern int link_ext_symbols;
 extern int force_pointer_size;
 extern int unwind_tables;
-extern int unix_lib;
+extern int use_dlltool;
+extern int use_msvcrt;
+extern int native_arch;
+extern int safe_seh;
+extern int prefer_native;
+extern int data_only;
 
 extern char *input_file_name;
 extern char *spec_file_name;
 extern FILE *output_file;
 extern const char *output_file_name;
 
-extern struct strarray lib_path;
 extern struct strarray tools_path;
 extern struct strarray as_command;
 extern struct strarray cc_command;
@@ -378,8 +389,6 @@ extern struct strarray nm_command;
 extern char *cpu_option;
 extern char *fpu_option;
 extern char *arch_option;
-extern const char *float_abi_option;
-extern int thumb_mode;
 extern int needs_get_pc_thunk;
 
 #endif  /* __WINE_BUILD_H */
